"""Example program for opening cards using aliases defined in Resource Manager"""

import pilxi

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Alias of the device we want to use as defined in the Resource Manager
    alias = "MyAlias"

    # Subunit we want to use
    subunit = 1

    try:
        # Open card by alias; this also establishes a session with the LXI device
        card = pilxi.Pi_Card_ByAlias(alias)

        subtype = card.SubType(subunit)

        print("Opened card subunit of type:", subtype)

        # Close the card. Closing a card object of class Pi_Card_ByAlias will also
        # close the LXI session.
        card.Close()

    # On error, print error message and exit
    except pilxi.Error as ex:
        print("Error occurred:", ex.message)
        exit()


import pilxi
import time

def main():
    # Session configuration

    IP_Address = '192.168.11.155'
    # Card and channel configuration values (adjust as needed)
    bus = 53        # Bus number from GSFP/Device Manager
    slot = 14        # Slot number from GSFP/Device Manager
    channel = 16     # Channel to which the signal is applied

    # Open a session to to LXI. Using "pxi" as ip address will access PXI connected to the computer.
    try:
        session = pilxi.Pi_Session(IP_Address)
    except Exception as e:
        print("Failed to create session:", e)
        return

    # Open the card.
    try:
        card = session.OpenCard(bus, slot)
    except Exception as e:
        print("Failed to open card:", e)
        return

    print("\n---------------------------PXI Comparator application---------------------------\n")
    try:
        card_id = card.CardId()  # Retrieve card ID
        print("-------------- Card ID:", card_id, "--------------")
    except Exception as e:
        print("Failed to retrieve CardId:", e)

    # ----------------------------- Channel configuration ------------------------------ #
    # Set channel attributes.
    # It is assumed that the following constants exist in the piplx module.
    try:
        # Set polarity (CMP_POL_UNI = 1 for Unipolar Mode)
        polarity = pilxi.CMP_Polarity.CMP_POL_UNI
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_POLARITY, polarity)
    except Exception as e:
        print("CMP_POLARITY failed:", e)

    try:
        # Set physical trigger mode (CMP_EDGE_EITHER = 3)
        phy_trig_mode = pilxi.CMP_Mode.CMP_EDGE_EITHER
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_TRIG_MODE, phy_trig_mode)
    except Exception as e:
        print("CMP_PHY_TRIG_MODE failed:", e)

    try:
        # Set virtual trigger mode (CMP_EDGE_DISABLED = 0)
        vir_trig_mode = pilxi.CMP_Mode.CMP_EDGE_DISABLED
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_TRIG_MODE, vir_trig_mode)
    except Exception as e:
        print("CMP_VIR_TRIG_MODE failed:", e)

    try:
        # Select logical operation mode: CMP_VIR_OR (0 for OR operation)
        vir_op = pilxi.CMP_Virt.CMP_VIR_OR
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_OR_AND, vir_op)
    except Exception as e:
        print("CMP_VIR_OR_AND failed:", e)

    try:
        # Set range to CMP_RANGE_18V (value 7 as defined)
        range_val = pilxi.CMP_Range.CMP_RANGE_18V
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_RANGE, range_val)
    except Exception as e:
        print("CMP_RANGE failed:", e)

    try:
        # Set physical mask (0 means none selected)
        phy_mask = 0
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_MASK, phy_mask)
    except Exception as e:
        print("CMP_PHY_MASK failed:", e)

    try:
        # Set virtual mask (0 means none selected)
        vir_mask = 0
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_MASK, vir_mask)
    except Exception as e:
        print("CMP_VIR_MASK failed:", e)

    try:
        # Set debounce time to 50.0 milliseconds
        debounce_time = 50.0
        card.SetAttributeDouble(channel, False, pilxi.Attributes.CMP_DEBOUNCE_TIME, debounce_time)
    except Exception as e:
        print("CMP_DEBOUNCE_TIME failed:", e)

    try:
        # Set threshold to 5.0 V (default was 10V in the original comments)
        threshold = 5.0
        card.SetAttributeDouble(channel, False, pilxi.Attributes.CMP_THRESHOLD, threshold)
    except Exception as e:
        print("CMP_THRESHOLD failed:", e)

    # ----------------------------- Read back configured values ------------------------------ #
    try:
        polarity_rbk     = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_POLARITY)
        phy_trig_mode_rbk = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_TRIG_MODE)
        vir_trig_mode_rbk = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_TRIG_MODE)
        vir_op_rbk       = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_OR_AND)
        range_rbk        = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_RANGE)
        phy_mask_rbk     = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_MASK)
        vir_mask_rbk     = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_MASK)
        debounce_rbk     = card.GetAttributeDouble(channel, False, pilxi.Attributes.CMP_DEBOUNCE_TIME)
        threshold_rbk    = card.GetAttributeDouble(channel, False, pilxi.Attributes.CMP_THRESHOLD)
    except Exception as e:
        print("Error reading back attribute:", e)

    print("Readback Data Channel:", channel)
    print("Phy trig. mode:", phy_trig_mode_rbk)
    print("Vir trig. mode:", vir_trig_mode_rbk)
    print("Logical op:", vir_op_rbk)
    print("Phy Mask:", phy_mask_rbk)
    print("Vir Mask:", vir_mask_rbk)
    print("Polarity:", polarity_rbk)
    print("Range:", range_rbk)
    print("Debounce time: {:.4f} ms".format(debounce_rbk))
    print("Threshold: {:.4f} V".format(threshold_rbk))
    print()

    # ----------------------------- Capture Engine Setup --------------------------------- #
    try:
        # Front Panel physical mask (all channels enabled)
        fp_phy_mask = 0xFFFF
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_FPT_MASK, fp_phy_mask)
    except Exception as e:
        print("CMP_PHY_FPT_MASK failed:", e)

    try:
        # Front Panel virtual mask (all channels enabled)
        fp_vir_mask = 0xFFFF
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_FPT_MASK, fp_vir_mask)
    except Exception as e:
        print("CMP_VIR_FPT_MASK failed:", e)

    try:
        # Reset the front panel trigger
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_FPT_RESET, 0)
    except Exception as e:
        print("CMP_FPT_RESET failed:", e)

    try:
        # Set capture append mode to disabled (new events overwrite previous data)
        capture_append = 0
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_APPEND, capture_append)
    except Exception as e:
        print("CMP_CAPTURE_APPEND failed:", e)

    # Allow time delay equal to debounce time before enabling capture
    time.sleep(debounce_time / 1000.0)

    try:
        # Enable the capture engine (1 = enable)
        capture_enable = 1
        card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_ENABLE, capture_enable)
    except Exception as e:
        print("CMP_CAPTURE_ENABLE failed:", e)

    print("\nCapturing Events Enabled")
    print("Press 's' to get current state of the card.")
    print("Press any other key to disable capturing and display recorded events.\n")

    disp_columns = False
    while True:
        ch = input("Press a key (s for status, any other to quit): ")
        if ch.lower() == 's':
            try:
                phy_state = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_PHY_STATE)
                vir_state = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_VIR_STATE)
                time_stamp_current = card.GetAttributeDWORDArray(channel, False, pilxi.Attributes.CMP_TIME_STAMP,2)
                index = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_INDEX)
            except Exception as e:
                print("Error retrieving card state:", e)
                continue

            if not disp_columns:
                print("--------------Current State Data--------------")
                print("Physical State     Virtual State    Index    Timestamp")
                disp_columns = True

            # Display states; hexadecimal formatting is used for bit masks
            print("{:<18} {:<16} {:<8} {}".format(hex(phy_state), hex(vir_state), index, time_stamp_current))
            continue

        # ------------------------ Disable Capture and read events ------------------------ #
        try:
            capture_enable = 0
            card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_ENABLE, capture_enable)
        except Exception as e:
            print("Disabling capture failed:", e)

        try:
            index = card.GetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_INDEX)
            read_offset = 0
            card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_READ_OFFSET, read_offset)
            time_stamp_ref = card.GetAttributeDWORDArray(channel, False, pilxi.Attributes.CMP_TIME_STAMP_REF,2)
        except Exception as e:
            print("Error fetching capture parameters:", e)
            index = 0

        if index:
            print("\n---------------------------RECORDED EVENTS---------------------------\n")
            print("Number of recorded events:", index)
            print("Time Stamp Ref:", time_stamp_ref)
            print("Raw Phy Data    Raw Vir Data    Event Phy Data    Event Vir Data    Timestamp")
        else:
            print("\nNO EVENTS RECORDED\n")

        # Loop over each recorded event (each call returns an array of 4 DWORD values)
        for i in range(read_offset, index):
            try:
                data_read = card.GetAttributeDWORDArray(channel, False, pilxi.Attributes.CMP_READ_DDR3,4)
            except Exception as e:
                print("Error reading event data:", e)
                break
            # Data layout interpretation (assuming a 4-element array)
            raw_phy_data   = (data_read[3] >> 16) & 0xFFFF
            raw_vir_data   = data_read[3] & 0xFFFF
            event_phy_data = (data_read[2] >> 16) & 0xFFFF
            event_vir_data = data_read[2] & 0xFFFF
            # Combine a 48-bit timestamp from two DWORDs
            time_stamp_event = (data_read[1] << 32) | data_read[0]
            print("{:<16} {:<16} {:<18} {:<18} {}".format(
                hex(raw_phy_data), hex(raw_vir_data),
                hex(event_phy_data), hex(event_vir_data), time_stamp_event))
        
        ch = input("Press 'y' to restart capture engine, any other key to quit: ")
        if ch.lower() == 'y':
            try:
                capture_enable = 1
                card.SetAttributeDWORD(channel, False, pilxi.Attributes.CMP_CAPTURE_ENABLE, capture_enable)
                print("\nCapturing Events Re-enabled\n")
            except Exception as e:
                print("Restarting capture engine failed:", e)
        else:
            break

    # -------------------- Reset and close the card --------------------- #
    try:
        card.ClearCard()
    except Exception as e:
        print("Clearing card failed:", e)

    try:
        card.Close()
    except Exception as e:
        print("Closing card failed:", e)

if __name__ == "__main__":
    main()




from __future__ import print_function
import pilxi


""" Sample program for Pickering LXI/PXI Thermocouple Simulator cards using the PILXI ClientBridge Python Wrapper """

if __name__ == "__main__":

    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.
    print("pilxi wrapper version: {}".format(pilxi.__version__))

    IP_Address = input("Please enter the IP address: ")

    # Default port and timeout settings in mS
    port = 1024
    timeout = 1000

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 1
    device = 12

    # Use the first subunit
    subunit = 1

    print("Sample program for Pickering LXI/PXI Thermocouple Simulator cards using the PILXI ClientBridge Python Wrapper")

    try:

        # Open a session with the LXI unit
        session = pilxi.Pi_Session(IP_Address)

        # Open a card by bus and device numbers
        card = session.OpenCard(bus, device)

        # Get the card ID
        cardId = card.CardId()

    # On any errors, print a description of the error and exit
    except pilxi.Error as ex:
        print("Error occurred: ", ex.message)
        exit()

    print("Connected to chassis at", IP_Address)
    print("Successfully connected to card at bus", bus, "device", device)
    print("Card ID: ", cardId)

    # Thermocouple specific functions:

    # Set subunit voltage range to auto
    print("Setting range to auto...")

    range = card.TS_RANGE["AUTO"]
    card.VsourceSetRange(subunit, range)

    # Get voltage range of a subunit
    range = card.VsourceGetRange(subunit)

    if range == card.TS_RANGE["AUTO"]:
        print("Set subunit", subunit, "range to auto.")

    # Set voltage to 19.5 mV on the subunit
    mvolts = 19.5

    print("Setting voltage to", mvolts, "mV...")

    card.VsourceSetVoltage(subunit, mvolts)

    # Read the voltage of a subunit
    mvolts = card.VsourceGetVoltage(subunit)

    print("Voltage set to", mvolts, " mV.")

    # 41-760-001 Example; Setting channel isolation switches
    # Please refer to your thermocouple manual to find the subunit for
    # your specific isolation switch subunit.
    isolation_subunit = 33

    if cardId.startswith("41-760-001"):
        # Turn Vo1 on
        card.OpBit(isolation_subunit, 1, 1)

        # Turn Vcold1 on
        card.OpBit(isolation_subunit, 2, 1)

        # Turn Vo1 off
        card.OpBit(isolation_subunit, 1, 0)

        # Turn Vcold1 off
        card.OpBit(isolation_subunit, 2, 0)

    # Get compensation block temperatures

    temperatures = card.VsourceGetTemperature(card.ATTR["TS_TEMPERATURES_C"])

    index = 0
    for index, temperature in enumerate(temperatures):
        print("Compensation block temperature", index, ":", temperature, "C")




"""Sample program for Pickering LXI 41-625 Function Generator"""

"""Sample program for Pickering LXI 41-625 Function Generator"""

import sys

import pilxi

###############################
# Initialize session and card #
###############################

IP_Address = "192.168.11.162"

port = 1024
timeout = 1000

bus = 4
device = 1

subunit = 1

waveform = pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE

# Connect to chassis
session = pilxi.Pi_Session(IP_Address, port, timeout)

# Open card
try:
    card = session.OpenCard(bus, device)
except pilxi.Error as ex:
    print("Error occurred: {}".format(ex.message))
    exit()

# Get and print the Card ID
cardId = card.CardId()

print("Successfully connected to card at bus", bus, "device", device)
print("Card ID: ", cardId)

#########################################
# Function generator specific functions #
#########################################

card.PIFGLX_SetWaveform(subunit, pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE)

card.PIFGLX_SetAmplitude(subunit, 20)

card.PIFGLX_SetFrequency(subunit, 10000)

card.PIFGLX_SetDcOffset(subunit, 0)

do_run = 1
amplitude = 20
freq = 10000
dc_offset = 0
while(do_run):
    amplitude = int(input("Set new amplitude"))
    freq = int(input("Set new frequency"))
    dc_offset = int(input("Set new DC Offset"))

    if (sys.version_info > (3, 0, 0)):
        amplitude = input("Set new amplitude")
    else:
        amplitude = raw_input("Set new amplitude")

    if (sys.version_info > (3, 0, 0)):
        freq = int(input("Set new frequency"))
    else:
        freq = int(raw_input("Set new frequency"))

    if (sys.version_info > (3, 0, 0)):
        dc_offset = int(input("Set new DC Offset"))
    else:
        dc_offset = int(raw_input("Set new DC Offset"))

    if(amplitude != None):
        card.PIFGLX_SetAmplitude(subunit, amplitude)
    if(freq != None):
        card.PIFGLX_SetFrequency(subunit, freq)
    if(amplitude != None):
        card.PIFGLX_SetDcOffset(subunit, dc_offset)

    if (sys.version_info > (3, 0, 0)):
        do_run = int(input("Enter 0 to abort generation"))
    else:
        do_run = int(raw_input("Enter 0 to abort generation"))

######################
# Clean up and close #
######################
card.PIFGLX_AbortGeneration(subunit)

card.ClearCard()

card.Close()

session.Close()


from __future__ import print_function
import pilxi

IPaddr = "192.168.11.12"

session = pilxi.Pi_Session(IPaddr)

sessions = session.GetForeignSessions()

for s in sessions:
    session.ReleaseForeignSession(s)

session.Disconnect()



"""Sample program for Pickering LXI 41-625 Function Generator"""

"""Sample program for Pickering LXI 41-625 Function Generator"""

import sys

import pilxi
import os

###############################
# Initialize session and card #
###############################

IP_Address = "192.168.11.156"

port = 1024
timeout = 1000

bus = 4
device = 13

subunit = 1

waveform = pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE

# Connect to chassis
session = pilxi.Pi_Session(IP_Address, port, timeout)

# Open card
try:
    card = session.OpenCard(bus, device)
except pilxi.Error as ex:
    print("Error occurred: {}".format(ex.message))
    exit()

# Get and print the Card ID
cardId = card.CardId()

print("Successfully connected to card at bus", bus, "device", device)
print("Card ID: ", cardId)

sample_source = ('{}/extras/MyARB.txt', os.getcwd())

card.PIFGLX_SetFrequency(subunit, 10000)
card.PIFGLX_CreateArbitraryWaveform(subunit, sample_source)

# Wait for user input to abort generation

if (sys.version_info > (3, 0, 0)):
    input_var = input("Press Enter to continue ")
else:
    input_var = raw_input("Please Enter to continue ")

######################
# Clean up and close #
######################
card.PIFGLX_AbortGeneration(subunit)

card.ClearCard()

card.Close()

session.Close()


"""Example program for using Pickering PXI resolver cards in LXI units using the Python ClientBridge wrapper"""

import pilxi

DM_Mode = pilxi.DM_Mode

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    IP_Address = "192.168.0.238"

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 1
    device = 14

    try:
        # Open a session with LXI device
        session = pilxi.Pi_Session(IP_Address)

        # Open a card using bus and device numbers
        card = session.OpenCard(bus, device)

    except pilxi.Error as ex:
        print("Error occurred: {}".format(ex.message))
        exit()

    # This function returns information about the card,
    # Model name, Serial number and Firmware revision
    cardId = card.CardId()

    print("Successfully connected to specified card.")
    print("Card ID: ", cardId)

    # Switch Sub-unit
    switchSub = 5
    bankOffset = 11

    # LVDT Bank (Sub-Unit) number
    bank = 2

    #Input Switches
    try:
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 1, True)
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 2, True)
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 3, True)

    except pilxi.Error as ex:
        print("Error occurred switching: {}".format(ex.message))
        exit()

    #Output Switches
    try:
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 5, True)
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 6, True)
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 8, True)
        card.OpBit(switchSub, ((bank - 1) * bankOffset) + 9, True)

    except pilxi.Error as ex:
        print("Error occurred switching: {}".format(ex.message))
        exit()

    try:
        mode = DM_Mode.RESOLVER  # Setting Resolver Mode
        card.VDTSetMode(bank, mode)

    except pilxi.Error as ex:
        print("Error occurred setting VDT mode: {}".format(ex.message))
        exit()

    # Set position
    try:
        position = -156.25 # Between -180.00 to 180.00
        card.ResolverSetPosition(bank, position)

    except pilxi.Error as ex:
        print("Error occurred setting position: {}".format(ex.message))
        exit()

    # Set Speed
    try:
        speed = 50.00 # Set speed between 1.00 - 655.35 RPM
        card.ResolverSetRotateSpeed(bank, speed)

    except pilxi.Error as ex:
        print("Error occurred setting speed: {}".format(ex.message))
        exit()

    # Set Number of Turns
    try:
        turns = 20 # Set number of turns between 1- 65535
        card.ResolverSetNumOfTurns(bank, turns)

    except pilxi.Error as ex:
        print("Error occurred setting number of turns: {}".format(ex.message))
        exit()

    # Start/Stop (Start = True, Stop = False)
    try:
        card.ResolverSetStartStopRotate(bank, True)

    except pilxi.Error as ex:
        print("Error occurred stopping/starting: {}".format(ex.message))
        exit()

    input("Press Enter to Stop...")
    card.ResolverSetStartStopRotate(bank, False)






from __future__ import print_function
import pilxi
import sys

""" Sample program for Pickering LXI/PXI 40-584-001 2x128 Matrix cards using PILXI ClientBridge Python Wrapper """

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.
    
    IP_Address = "PXI"

    # Default port and timeout settings in mS
    port = 1024
    timeout = 1000

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 21
    device = 0

    # Use the first subunit
    subunit = 1

    print("Sample program for Pickering LXI/PXI 40-584-001 2x128 Matrix cards using PILXI ClientBridge Python Wrapper")
    print("Connecting to chassis at ", IP_Address)

    # Open a session with LXI device
    session = pilxi.Pi_Session(IP_Address, port, timeout)

    try:
        # Open the card
        card = session.OpenCard(bus, device)

    except pilxi.Error as ex:
        print("Error occurred: {}".format(ex.message))
        exit()

    # Get the card ID and an error code
    cardId = card.CardId()

    print("Successfully connected to card at bus", bus, "device", device)
    print("Card ID: ", cardId)

    print("Clearing subunit", subunit)
    # card.ClearSub() de-energises all outputs on a specified subunit
    card.ClearSub(subunit)

    # SetCrosspointRange sets a range of crosspoints in a given row   
    # Accepting the Row from user.
    if (sys.version_info > (3, 0, 0)):
        input_var = input("Please enter the Row (Y): ")
    else:
        input_var = raw_input("Please enter the Row (Y): ")
    row = int(input_var)

    # Accepting the starting value of the column range from user.
    if (sys.version_info > (3, 0, 0)):
        input_var = input("Please enter the START value of column range to be controlled (X): ")
    else:
        input_var = raw_input("Please enter the START value of column range to be controlled (X): ")
    start_column = int(input_var)
    
    # Accepting the ending value of the column range from user.
    if (sys.version_info > (3, 0, 0)):
        input_var = int(input("Please enter the END value of column range to be controlled (X): "))
    else:
        input_var = raw_input("Please enter the END value of column range to be controlled (X): ")
    end_column = int(input_var)

    print("Switching range", start_column, "-", end_column, "on row", row)

    state = 1 # to energize the crosspoints
    #state = 0 # to de energize the crosspoints
    card.SetCrosspointRange(subunit, row, start_column, end_column, state)

    if (sys.version_info > (3, 0, 0)):
        input_var = input("Press Enter to continue ")
    else:
        input_var = raw_input("Please Enter to continue ")

    print("Clearing outputs and closing card...")
    card.ClearSub(subunit)

    # Close the card before exiting the program
    card.Close()

    # Close the LXI session
    session.Close()



"""Example program for using the Pickering 41-620 Function Generator card with the Python pi620lx ClientBridge wrapper."""

import pilxi
import pi620lx

if __name__ == "__main__":

    print("pi620lx wrapper version: {}".format(pi620lx.__version__))
    
    # IP address of the LXI
    IP_Address = "192.168.0.244"

    # Open a session with the LXI unit and get the session ID
    session = pilxi.Pi_Session(IP_Address)
    sessionID = session.GetSessionID()

    # Open the pi620lx library using the LXI session
    pi620Base = pi620lx.Base(sessionID)

    # Discover and list 41-620 cards present in LXI
    cards = pi620Base.findCards()

    for card in cards:
        bus, device = card
        print("Found 41-620 card at bus {} device {}".format(bus, device))

    # Open a 41-620 card. With no parameters, the openCard() method will open the first 41-620 found.
    # This is useful if your LXI has only one 41-620 function generator card present.
    # Otherwise, the method can be called as below:
    # card = pi620Base.openCard(bus, device)
    # using bus and device numbers obtained from the Base.findCards() method
    card = pi620Base.openCard()

    # Set active channel to use
    channel = 1
    card.setActiveChannel(channel)

    # Switch off channel output before configuring it
    card.outputOff()

    # Set trigger mode to continuous (no trigger)
    card.setTriggerMode(card.triggerSources["FRONT"], card.triggerModes["CONT"])

    # Set DC offset to generated waveform (float value from -5 to 5 volts)
    # The first argument specifies the desired offset voltage;
    # the second enables or disables DC offset.
    offsetVoltage = 4.0
    enableDCOffset = True
    card.setOutputOffsetVoltage(offsetVoltage, enableDCOffset)

    # Set attenuation to signal amplitude (float value in dB)
    attenuation = 3
    card.setAttenuation(attenuation)

    # Generate a signal
    # Signal shape can be defined using constants available with the Pi620_Card class:
    shape = card.signalShapes["SINE"]
    # shape = card.signalShapes["SQUARE"]
    # shape = card.signalShapes["TRIANGLE"]

    # Frequency of signal in kHz:
    frequency = 1
    # Symmetry of signal (0 - 100):
    symmetry = 20

    try:
        # Start generating a signal. By default, this method will start generating immediately without
        # first calling card.outputOn().
        # card.generateSignal(frequency, shape, symmetry)

        # The card.generateSignal() method can also be used with optional parameters to specify
        # a start phase offset and to enable/disable immediate signal generation.
        # For example, the following call will set the same signal as above, but with a
        # 90 degree phase offset and will disable signal output until card.outputOn() is called:
        card.generateSignal(frequency, shape, symmetry, startPhaseOffset=90, generate=False)

        # Set output on
        card.outputOn()

    except pi620lx.Error as error:
        print("Exception occurred:", error.message)

    # Close card. This will not stop the card generating a signal.
    card.close()




from __future__ import print_function
import pilxi

if __name__ == "__main__":

    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    print("pilxi wrapper version: {}".format(pilxi.__version__))    
    IP_Address = "192.168.0.238"

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 1
    device = 14

    try:
        # Open a session with LXI device
        session = pilxi.Pi_Session(IP_Address)

        # Open a card using bus and device numbers
        card = session.OpenCard(bus, device)

    except pilxi.Error as ex:
        print("Error occurred: {}".format(ex.message))
        exit()

    print("Sample program for manipulating subunit states on Pickering Matrix Cards using the PILXI Python Wrapper")
    print()

    # This function returns information about the card,
    # Model name, Serial number and Firmware revision
    cardId = card.CardId()

    print("Successfully connected to specified card.")
    print("Card ID: ", cardId)

    # Use the 1st subunit
    subunit = 1

    # This function returns the number of rows and columns of the specified card subunit
    typeNum, rows, columns = card.SubInfo(subunit, True)
    subType = card.SubType(subunit, 1)
    print("Rows: {}\nColumns: {}".format(rows, columns))
    print("Card subunit type: {}".format(subType))
    print()

    # This function clears (de-energises) all outputs of a subunit
    card.ClearSub(subunit)

    # Get the subunit state object
    subState = card.GetSubState(subunit)

    # If we're using a matrix subunit:
    if subType.startswith("MATRIX"):

        # Switch crosspoints on the matrix
        row = 1
        column = 1
        subState.PreSetCrosspoint(row, column, True)

        row = 1
        column = 2
        subState.PreSetCrosspoint(row, column, True)

    # If we're using a switch subunit
    elif subType.startswith("SWITCH"):

        # Pre-set a range of bits
        for bit in range(1, 5):
            subState.PreSetBit(bit, 1)

    input("Press Enter to continue...")
    print()

    try:
        # Write the changed state back to the card
        card.WriteSubState(subunit, subState)
        print("Wrote modified state to subunit", subunit)

    except pilxi.Error as ex:
        print("Error occurred writing subunit state: {}".format(ex.message))
        exit()

    # Close the card
    card.Close()

    # Close the LXI session
    session.Close()



# Example program for Pickering 41-765-001 PXI Current Loop simulator
# Using the python-pilxi Python ClientBridge wrapper

import pilxi

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.
    IP_Address = "192.168.1.200"

    # Open a session with LXI device
    session = pilxi.Pi_Session(IP_Address)

    # Bus and device numbering for the 41-765-001
    bus = 4
    device = 14

    try:
        # Open the card
        card = session.OpenCard(bus, device)

    except pilxi.Error as ex:
        print("Error opening card: {}".format(ex.message))
        exit()

    # Get the card's ID string
    cardId = card.CardId()

    print("Successfully connected to card at bus", bus, "device", device)
    print("Card ID: ", cardId)

    try:
        # Sets 0 - 24 mA mode on subunit (channel) 1
        mode = card.CL_MODE["0_24_MA"]
        subunit = 1
        card.CLSetMode(subunit, mode)

        # Set current value to 6.55 mA on output
        current = 6.55
        card.CLSetCurrent(subunit, current)

        # Set 0 - 5V mode on subunit (channel) 1
        mode = card.CL_MODE["0_5_V"]
        card.CLSetMode(subunit, mode)

        # Sets voltage value to 2.25V on output
        voltage = 2.25
        card.CLSetVoltage(subunit, voltage)

    except pilxi.Error as ex:
        print("Error occurred: {}".format(ex.message))
        exit()

    # 40-765-001 specific example; setting isolation switches
    # Isolation switches are organised as subunit 17
    # Please refer to your product manual as this may vary between models.

    try:
        isolationSubunit = 17
        isolationSwitch = 1
        state = 1

        # Close Channel 1 isolation switch
        card.OpBit(isolationSubunit, isolationSwitch, state)

    except pilxi.Error as ex:
        print("Error operating isolation switches: {}".format(ex.message))
        exit()

    # Close the card and session
    card.Close()
    session.Close()




"""Sample program for Pickering LXI 41-625 Function Generator"""

import pilxi

################
# Channel mask #
#    1 = ON    #
#    0 = OFF   #
################

# set first 4 channels to 1
channel_state = [
    1,1,1,1
]

# fill the rest with 0
for channel in range(len(channel_state), 32):
    channel_state[channel] = 0

############################
# Variables for generation #
############################

waveform_type = [
    pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE,
    pilxi.WaveformTypes.PIFGLX_WAVEFORM_SQUARE,
    pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE,
    pilxi.WaveformTypes.PIFGLX_WAVEFORM_SQUARE
]

amplitude = [
    25,
    15,
    25,
    15
]

frequency = [
    10000,
    10000,
    10000,
    10000
]

dc_offset = [
    0,
    2,
    0,
    2
]

# fill the rest with 0
for channel in range(len(channel_state), 32):
    channel_state[channel] = 0

###############################
# Initialize session and card #
###############################

IP_Address = "192.168.11.156"

port = 1024
timeout = 1000

bus = 4
device = 13

subunit = 1

# Connect to chassis
session = pilxi.Pi_Session(IP_Address, port, timeout)

# Open card
try:
    card = session.OpenCard(bus, device)
except pilxi.Error as ex:
    print("Error occurred: {}".format(ex.message))
    exit()

# Get and print the Card ID
cardId = card.CardId()

print("Successfully connected to card at bus", bus, "device", device)
print("Card ID: ", cardId)

#########################################
# Function generator specific functions #
#########################################

# count number of enabled channels
num_channels = 0
for x in range(len(channel_state)):
    if channel_state[x] == 1:
        num_channels += 1


for channel in range(num_channels):
    card.PIFGLX_SetWaveform(channel, waveform_type[channel])
    card.PIFGLX_SetAmplitude(channel, amplitude[channel])
    card.PIFGLX_SetFrequency(channel, frequency[channel])
    card.PIFGLX_SetDcOffset(channel, dc_offset[channel])





from __future__ import print_function
import pilxi
from LXIpxm78xx import __version__
from LXIpxm78xx import *

# IP address of the Pickering LXI chassis containing the PXM78xx instrument
LXI_IP = "192.168.10.228"

def example_DMM(instrument):

    ### Example: Operating instrument as DMM ###

    try:

        # Reset the instrument
        instrument.reset()

        # Initialise DMM
        instrument.DMM_init()

        # Configure DMM to DC voltage, range DC 4V
        instrument.DMM_configMeasurement(PXM78xx_FUNC.DMM_DCV, PXM78xx_RANGE.DCV_4V)

        # Configure filter
        instrument.DMM_configFilter(PXM78xx_FILTER.FILTER_1kHz)

        # Measure
        aperture = 0.1
        value, sbResult = instrument.DMM_meanMeasure(aperture)

        # Print results

        print("Result = ", value)
        print("Min = ", sbResult.min)
        print("Max = ", sbResult.max)

    except PXM78xx_Error as ex:

        print("Error operating instrument: ", ex.message)
        print("Error code ", hex(ex.errorCode))

def example_WFD(instrument):
    pre_samples = 1
    post_samples = 50
    total_samples = pre_samples + post_samples
    ### Example: Operating instrument as waveform digitiser ###

    try:

        # Reset the instrument
        print("Reseting instrument . . . ", end='')
        instrument.reset()
        print("OK")

        # Prepare waveform digitiser mode
        print("WFD_init() . . . ", end='')
        instrument.WFD_init(PXM78xx_CHANNEL.ADC16BIT)
        print("OK")

        # Configure measurement function and range
        print("WFD_configMeasurement() . . . ", end='')
        instrument.WFD_configMeasurement(PXM78xx_FUNC.WFD_DCV, PXM78xx_RANGE.DCV_8V)
        print("OK")

        sampleCfg = WFD_SAMPLECFG()

        # Configure sampling (level trigger)
        sampleCfg.interval = 500e-5
        sampleCfg.postTrigger = pre_samples
        sampleCfg.preTrigger = post_samples
        sampleCfg.triggerDelay = 0.0
        sampleCfg.triggerHysteresis = 0.1
        sampleCfg.triggerLevel = 0.1
        sampleCfg.triggerSource = PXM78xx_WFD_TRIGGER.RISING
        sampleCfg.continueWithNext = 0
        sampleCfg.noWaitForTrigger = 1

        print("WFD_configSampling() . . . ", end='')
        instrument.WFD_configSampling(sampleCfg)
        print("OK")

        # Configure averaging
        print("WFD_configMeanCount() . . . ", end='')
        instrument.WFD_configMeanCount(1)
        print("OK")

        # Start sampling
        print("WFD_startSampling() . . . ", end='')
        instrument.WFD_startSampling()
        print("OK")

        # Configure number of samples
        sampleCount = total_samples

        print("WFD_querySampleData() . . . ", end='')
        xData, yData = instrument.WFD_querySampleData(sampleCount)
        print("OK")

        print("Sampled Data (level trigger):")

        for i in range(len(xData)):
            print((xData[i] * 1e6), " us ", yData[i], "V")

    except PXM78xx_Error as ex:
        print("Error operating instrument: ", ex.message)
        print("Error code ", hex(ex.errorCode))
        exit(1)

def main():

    ### Example: Connecting to LXI and opening instrument(s) ###

    try:
        print("LXIpxm78xx wrapper version: {}".format(__version__))
        # Open a session with the LXI and get a session ID to use with
        # the PXM78xx wrapper
        
        print("Connecting to " + LXI_IP + " . . . ", end = '')
        session = pilxi.Pi_Session(LXI_IP)
        print("OK")
        sessionID = session.GetSessionID()

        # PXM78xx base class is used to discover and open instruments
        base = PXM78xx_Base(sessionID)



        libvers = base.QueryLibraryVersions()
        print("Library versions: ")
        print(libvers)

        # Discover free instruments in LXI
        print("Searching for instruments . . . ", end='')
        instruments = base.findFreeInstruments()
        print("OK")

        print("Found ", len(instruments), " Instruments:")
        for instr in instruments:
            print("Instrument at: ", instr)
        print()

        # Auto connect to first instrument found
        # Instrument opening functions return an instrument object
        # all methods operating the instrument belong to this object
        print("Connecting to first instrument . . . ", end='')
        instrument = base.autoConnectToFirst()
        print("OK")

    except PXM78xx_Error as ex:

        print("Error occurred opening instrument: ", ex.message)
        print("Error code", hex(ex.errorCode))
        exit(1)

    # Query some information about the instrument
    print("Instrument name: ", instrument.queryProductName())
    print("Instrument serial number: ", instrument.querySerialNumber())
    print("Instrument status: ", instrument.queryStatusMessage())
    print()

    # Example for operating the instrument as a DMM;
    example_DMM(instrument)
    
    # Example for operating the instrument as a waveform digitiser
    example_WFD(instrument)


if __name__ == "__main__":
    main()



""" Sample program for Pickering LXI/PXI Precision Resistor cards using the PILXI ClientBridge Python Wrapper """

import pilxi

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    IP_Address = "192.168.0.12"

    # Default port and timeout settings in mS
    port = 1024
    timeout = 1000

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 7
    device = 12

    print("Sample program for Pickering LXI/PXI Precision Resistor cards using the PILXI ClientBridge Python Wrapper")

    # Open a session with LXI
    try:
        session = pilxi.Pi_Session(IP_Address, port, timeout)
    except pilxi.Error as ex:
        print("Opening LXI session failed: {}".format(ex.message))
        exit()

    # Open the card
    try:
        card = session.OpenCard(bus, device)
    except pilxi.Error as ex:
        print("Opening card failed: {}".format(ex.message))
        exit()

    # Get the card ID
    cardId = card.CardId()

    print("Connected to chassis at", IP_Address)
    print("Successfully connected to card at bus:", bus, "device:", device)
    print("Card ID: ", cardId)

    #use subunit 1 of the card.
    subunit = 1

    # Functions to control Precision Resistor:
    try:
        #Set Resistance
        #       ---/ ------------
        #       |               |
        # ----------------R----------
        resistance = 560
        card.ResSetResistance(subunit, resistance)

        #Get Resistance
        resistance = 0
        resistance = card.ResGetResistance(subunit)
        print("Resistance readback ", resistance)

        #short circuit
        #       -----------------
        #       |               |
        # ---------/  ----R----------
        resistance = 0
        card.ResSetResistance(subunit, resistance)

        #open circuit
        #       ---/ ------------
        #       |               |
        # ---------/  ----R----------
        resistance = float("inf")
        card.ResSetResistance(subunit, resistance)

    except pilxi.Error as ex:
        print("Error occurred operating Precision Resistor Card: {}".format(ex.message))
        print("Error code: {}".format(ex.errorCode))
    
    # Close the card. It is recommended to close any open cards
    print("Closing card and exiting.")

    card.Close()



from __future__ import print_function
import pilxi

if __name__ == "__main__": 
    
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    IP_Address = "192.168.7.75"
    
    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 4
    device = 15
    
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Open a session with LXI device
    session = pilxi.Pi_Session(IP_Address)

    print("Sample program for Pickering LXI/PXI Matrix cards using PILXI ClientBridge Python Wrapper")
    print("Connecting to chassis at ", IP_Address)
    
    try:
        # Open the card
        card = session.OpenCard(bus, device)

    except pilxi.Error as ex:
        print("Error opening card: {}".format(ex.message))
        exit()

    # Get the card ID and an error code
    cardId = card.CardId()

    print("Successfully connected to card at bus", bus, "device", device)
    print("Card ID: ", cardId)

    # Use the 1st subunit
    subunit = 1

    print("Clearing subunit", subunit)
    # card.ClearSub() de-energises all outputs on a specified subunit
    card.ClearSub(subunit)

    # Now we can set some crosspoints on the matrix card subunit

    row = 1
    column = 1
    state = True
    card.OpCrosspoint(subunit, row, column, state)

    row = 1
    column = 2
    state = True
    card.OpCrosspoint(subunit, row, column, state)

    # Close the card
    card.Close()

    # Close the LXI session
    session.Close()



""" Sample program for Pickering LXI/PXI Battery Simulator cards using the PILXI ClientBridge Python Wrapper """

import pilxi

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    IP_Address = "192.168.2.80"

    # Default port and timeout settings in mS
    port = 1024
    timeout = 1000

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 4
    device = 13

    # Use the first subunit
    subunit = 1

    print("Sample program for Pickering LXI/PXI Battery Simulator cards using the PILXI ClientBridge Python Wrapper")

    # Open a session with LXI
    try:
        session = pilxi.Pi_Session(IP_Address, port, timeout)
    except pilxi.Error as ex:
        print("Opening LXI session failed: {}".format(ex.message))
        exit()

    # Open the card
    try:
        card = session.OpenCard(bus, device)
    except pilxi.Error as ex:
        print("Opening card failed: {}".format(ex.message))
        exit()

    # Get the card ID
    cardId = card.CardId()

    print("Connected to chassis at", IP_Address)
    print("Successfully connected to card at bus:", bus, "device:", device)
    print("Card ID: ", cardId)

    # Functions to control Battery Simulator:

    volts = 3.3
    current = 0.03

    try:
        print("Setting voltage to", volts)
        # Set Voltage
        card.BattSetVoltage(subunit, volts)

        print("Setting current to", current)
        # Set Current
        card.BattSetCurrent(subunit, current)

        print("Enabling Battery Simulator output")

        # Enable Output
        card.BattSetEnable(subunit, 1)

        # Get Interlock State
        interlockState = card.BattReadInterlockState(subunit)
        if interlockState:
            print("Interlock is 'up' (enabled)")
        else:
            print("Interlock is 'down' (disabled)")

        # Get Voltage
        volts = card.BattGetVoltage(subunit)
        print("Voltage is set to ", volts)

        # Set Current
        current = card.BattGetCurrent(subunit)
        print("Current is set to", current)

        # Get Output state
        state = card.BattGetEnable(subunit)
        if state:
            print("Output state is enabled")

    except pilxi.Error as ex:
        print("Error occurred operating Battery Simulator: {}".format(ex.message))
        print("Error code: {}".format(ex.errorCode))

    try:
        # Measure channel voltage and current on supported battery simulator cards
        # e.g. 41-752A-01x models. Please consult your product manual for your
        # card's specific capabilities.

        # 41-752A-01x: Enable set-measure-set mode.
        card.BattSetMeasureSet(subunit, True)

        # 41-752A-01x: Change measurement device accuracy.
        card.BattSetMeasureConfig(subunit,                              # Subunit to configure
                                  pilxi.BattNumSamples.SAMPLES_128,     # Average values after 128 samples
                                  pilxi.BattConversionTime.T_1052us,    # 1052 us voltage sample time
                                  pilxi.BattConversionTime.T_540us,     # 540 us current sample time
                                  pilxi.BattOperationMode.CONTINUOUS)   # Measure continuously (no wait for trigger)

        # The battery simulator (41-752A-01x) has the capability to take into consideration the load
        # at which the voltage must be provided. Calculated data for voltage at different loads are
        # used to provide this functionality.
        load = 100  # units: mA
        card.BattSetLoad(subunit, load)

        volts = card.BattMeasureVoltage(subunit)
        print("Measured voltage: ", volts)

        current = card.BattMeasureCurrentmA(subunit)
        print("Measured ", current, " mA")

        current = card.BattMeasureCurrentA(subunit)
        print("Measured ", current, " A")

    except pilxi.Error as ex:
        print("Error operating special battery simulator functionality: {}".format(ex.message))
        print("Error code: {}".format(ex.errorCode))
        exit()

    # Close the card. It is recommended to close any open cards
    print("Closing card and exiting.")

    card.Close()



from __future__ import print_function

import math

import pilxi

"""Example Program for Pickering LXI-PXI-PCI DIO Mux type cards, using Clientbridge (piplx and picmlx) driverr"""

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))

    # IP Address to use.
    # To use local PXI cards, place 'PXI' as the IP address.
    IP_Address = "192.168.2.236"

    # Open a session with an LXI device.
    # If any errors occur, an exception will be raised.
    try:
        session = pilxi.Pi_Session(IP_Address)
        print("Opened session with LXI at", IP_Address)

    except pilxi.Error as ex:
        print("Error ocurred opening session:", ex.message)
        exit()

    # Open a card by bus and device numbers.
    try:
        bus = 3
        device = 14

        card = session.OpenCard(bus, device)
        print("Opened card at bus {} device {}".format(bus, device))
        print(card.CardId())
    except pilxi.Error as ex:
        print("Error ocurred opening card:", ex.message)
        exit()

    # Perform switching operations
    try:
        #subunit 3: 12 bit threshold value
        subunit = 3
        voltage = 0.5
        threshold = 4095 * (voltage / 50.0)
        threshold = math.trunc(threshold)
        #print formatting
        binaryThreshold = bin(threshold)
        #writeSub data type cast
        threshold = [threshold]

        card.ClearSub(subunit)
        print("\nSetting threshold 1 to", str(voltage) + "V with bit pattern", binaryThreshold.replace('0b', ''))
        card.WriteSub(subunit, threshold)

        #subunit 4: 12 bit threshold value
        subunit = 4
        voltage = 10
        threshold = 4095 * (voltage / 50.0)
        threshold = math.trunc(threshold)
        #print formatting
        binaryThreshold = bin(threshold)
        #writeSub data type cast
        threshold = [threshold]

        card.ClearSub(subunit)
        print("Setting threshold 2 to", str(voltage) + "V with bit pattern", binaryThreshold.replace('0b', ''))
        card.WriteSub(subunit, threshold)

        #subunit 5: specifies channel control (MUX) //40-412-001 only
        subunit = 5
        channel = 5
        state = 1
        card.OpBit(subunit, channel, state)
        print("\nSwitched subunit: {} bit: {} new_state: {}".format(subunit, channel, state))

        #read the input subunit 1: specific channel threshold readings
        subunit = 1
        print("\nFor channel", channel, "Input threshold states are:", int(card.ReadBit(subunit, 2)), int(card.ReadBit(subunit, 1)))

    except pilxi.Error as ex:
        print("Error:", ex.message)
        exit()

    # Close the card and close the session with the LXI unit.

    card.Close()
    session.Close()



"""Sample program for Pickering LXI 41-625 Function Generator"""

import sys

import pilxi

###############################
# Initialize session and card #
###############################

IP_Address = "192.168.11.162"

port = 1024
timeout = 1000

bus = 4
device = 1

subunit = 1

waveform = pilxi.WaveformTypes.PIFGLX_WAVEFORM_SINE

amplitude = 20
frequency = 10000
dc_offset = 0
start_phase = 0
duty_cycle = 20

# Pulse waveform only
pulse_width = 150

# Connect to chassis
session = pilxi.Pi_Session(IP_Address, port, timeout)

# Open card
try:
    card = session.OpenCard(bus, device)
except pilxi.Error as ex:
    print("Error occurred: {}".format(ex.message))
    exit()

# Get and print the Card ID
cardId = card.CardId()

print("Successfully connected to card at bus", bus, "device", device)
print("Card ID: ", cardId)

card.ClearCard()

#########################################
# Function generator specific functions #
#########################################

card.PIFGLX_SetInputTriggerConfig(pilxi.TriggerInputSource.PIFGLX_TRIG_IN_FRONT, pilxi.TriggerInputModes.PIFGLX_TRIG_IN_EDGE_RISING)

card.PIFGLX_SetOutputTriggerConfig(pilxi.TriggerOutputModes.PIFGLX_TRIG_OUT_GEN_PULSE_POS, 0)

card.PIFGLX_SetInputTriggerEnable(subunit, [1])

card.PIFGLX_SetOutputTriggerEnable(subunit, [1])

card.PIFGLX_SetWaveform(subunit, waveform)

card.PIFGLX_SetAmplitude(subunit, amplitude)

card.PIFGLX_SetDcOffset(subunit, dc_offset)

card.PIFGLX_SetFrequency(subunit, frequency)

card.PIFGLX_SetStartPhase(subunit, start_phase)

card.PIFGLX_SetDutyCycleHigh(subunit, duty_cycle)

if waveform == pilxi.WaveformTypes.PIFGLX_WAVEFORM_PULSE:
    card.PIFGLX_SetPulseWidth(pulse_width)

# Wait for user input to abort generation

if (sys.version_info > (3, 0, 0)):
    input_var = input("Press Enter to continue ")
else:
    input_var = raw_input("Please Enter to continue ")

######################
# Clean up and close #
######################
card.PIFGLX_AbortGeneration(subunit)

card.ClearCard()

card.Close()

session.Close()


from __future__ import print_function
import pilxi
import sys

""" Sample program for Pickering LXI/PXI Attenuator cards using the PILXI ClientBridge Python Wrapper """

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.
    IP_Address = "192.168.0.12"

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 13
    slot = 12

    # Use the first subunit
    subunit = 1

    print("Sample program for Pickering LXI/PXI Attenuator cards using the PILXI ClientBridge Python Wrapper")

    session = pilxi.Pi_Session(IP_Address)

    # Open a card at the given bus and device.
    try:
        card = session.OpenCard(bus, slot)

    # If an error occurs, an exception will be raised that will contain an error message
    # and the driver error code.
    except pilxi.Error as ex:
        print("Driver error {} occurred: {}".format(ex.errorCode, ex.message))
        exit()

    # Get the card ID and an error code
    cardId = card.CardId()

    print("Connected to chassis at", IP_Address)
    print("Successfully connected to card at bus", bus, "device", slot)
    print("Card ID: ", cardId)

    # Attenuation values are expressed in decibels (dB):
    attenuation = 3

    try:
        # This function sets attenuation value on the specified subunit
        card.SetAttenuation(subunit, attenuation)
        print("Set attenuation value to", attenuation, "dB...")

        # This function returns the current attenuation value:
        attenuation = card.GetAttenuation(subunit)
        print("Got attenuation value of ", attenuation, "dB.")

    except pilxi.Error as ex:
        print("Driver error {} occurred: {}".format(ex.errorCode, ex.message))
        exit()

    # Close the card. It is recommended to close any open cards
    card.Close()

    # Close the session with LXI device
    session.Close()



import pilxi

print("Example program to list cards in a Pickering LXI chassis")
print("pilxi wrapper version: {}".format(pilxi.__version__))
print()

IP_Address = "192.168.2.80"

session = pilxi.Pi_Session(IP_Address)

cards = session.FindFreeCards()
index = 1

if len(cards) > 0:
    print("Found cards:")

    for cardLocation in cards:

        bus, device = cardLocation
        card = session.OpenCard(bus, device)

        cardID = card.CardId()

        print(f"Card {index}: bus {bus} device {device}: {cardID}")

        index += 1

else:
    print("Found no cards.")



from __future__ import print_function
import pilxi

"""Example program for operating Pickering Switch cards using the Python-Pilxi wrapper"""

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # IP Address to use.
    # To use local PXI cards, place 'PXI' as the IP address.
    IP_Address = "10.0.0.27"

    # Open a session with an LXI device.
    # If any errors occur, an exception will be raised.
    try:
        session = pilxi.Pi_Session(IP_Address)
        print("Opened session with LXI at", IP_Address)

    except pilxi.Error as ex:
        print("Error ocurred opening session:", ex.message)
        exit()

    # Open a card by bus and device numbers.
    try:
        bus = 0
        device = 4

        card = session.OpenCard(bus, device)
        print("Opened card at bus {} device {}".format(bus, device))

    except pilxi.Error as ex:
        print("Error ocurred opening card:", ex.message)
        exit()

    # Perform switching operations
    try:
        subunit = 1
        bitNumber = 1
        state = 1

        card.OpBit(subunit, bitNumber, state)
        print("Switched subunit: {} bit: {} state: {}".format(subunit, bitNumber, state))

    except pilxi.Error as ex:
        print("Error switching:", ex.message)
        exit()

    # Close the card and close the session with the LXI unit.

    card.Close()
    session.Close()








from __future__ import print_function
import pilxi

# Sample program for Pickering LXI/PXI Strain Gauge Simulator cards (ex: 40-265-016) using the PILXI ClientBridge Python Wrapper
# This example is written for the Pickering 40-265-016 6-channel Strain Gauge simulator card. Modification to the source code
# may be required for other cards. Please refer to your product manual for information on how to operate your specific card. 

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # Connect to a chassis using an IP address.
    # The ClientBridge driver can also connect to local PXI chassis by passing
    # 'PXI' in place of the IP.

    IP_Address = "192.168.0.12"

    # Default port and timeout settings in mS
    port = 1024
    timeout = 1000

    # In this example we'll directly connect to the card using bus and device numbers:
    bus = 13
    device = 13

    # The Pickering 40-265-016 6-channel Strain Gauge simulator card uses a set of Precision Resistor subunits
    # numbered 1 - 6, followed by a set of Bridge Control (switch) subunits.
    # Bridge Control subunits on the 6-channel card are numbered n + 6, where n is the number of the
    # corresponding Precision Resistor subunit. 4-channel cards will have Bridge Control subunits at n + 4, and so on.
    # Please refer to your product manual for more information on your specific card.

    # In this example we'll use the first resistor subunit and its corresponding Bridge Control switch subunit:
    subunit = 1
    switchSubunit = subunit + 6

    try:

        # Open a session with the LXI unit
        session = pilxi.Pi_Session(IP_Address)

        # Open a card by bus and device numbers
        card = session.OpenCard(bus, device)

        # Get the card ID
        cardId = card.CardId()

    # On any errors, print a description of the error and exit
    except pilxi.Error as ex:
        print("Error occurred: ", ex.message)
        exit()

    print("Connected to chassis at", IP_Address)
    print("Successfully connected to card at bus", bus, "device", device)
    print("Card ID: ", cardId)

    # Controlling the 40-265-016 6-channel Strain Gauge simulator
    # Bridge control subunit bits for this card are as follows:
    # Bit 1 - Internal Excitation
    # Bit 2 - External Excitation
    # Bit 3 - Output Enable
    # Bit 4 - Bridge Switches
    BIT_INTERNAL_EXCITATION = 1
    BIT_EXTERNAL_EXCITATION = 2
    BIT_OUTPUT_ENABLE       = 3
    BIT_BRIDGE_SWITCHES     = 4

    # Please refer to your product manual for specific information about your card.

    # Switch Bridge Control subunit excitation source to external excitation:
    card.OpBit(switchSubunit, BIT_EXTERNAL_EXCITATION, 1)

    # Set resistor output to a specified value in Ohms:
    resistance = 350.503
    card.ResSetResistance(subunit, resistance)

    # Get resistance
    resistance = card.ResGetResistance(subunit)
    print("Resistance set to", resistance)

    # Set output enable switch
    card.OpBit(switchSubunit, BIT_OUTPUT_ENABLE, 1)

    # It is recommended to close open cards after using them
    card.Close()


# Example for multiuser LXI access using the Pickering pilxi Python Clientbridge wrapper
# Two instances of this script can be run, demonstrating multithreaded/multiuser access
# to an LXI switch card.

import pilxi

if __name__ == "__main__":

    print("Python Clientbridge multiuser access demonstration program")
    print("pilxi wrapper version: {}".format(pilxi.__version__))

    IP_Address = "192.168.2.83"

    try:
        # Connect to LXI and get a session object
        session = pilxi.Pi_Session(IP_Address)

    except pilxi.Error as ex:
        print(f"Error getting session: {ex.message}")
        print("Please check your connection and IP address.")
        exit()

    # Get a list of usable cards
    cardsList = session.GetUsableCards(0)

    print(f"Found {len(cardsList)} usable card(s):")

    # Open the first card for multiuser access
    try:
        card = session.OpenCardByID(cardsList[0], accessType=pilxi.AccessTypes.MULTIUSER)

    except pilxi.Error as ex:
        print(f"Error opening card: {ex.message}")
        exit()

    subunitType = card.SubType(1)

    print(f"Card Type: {subunitType}")
    print()

    print("Card switch state will be repeatedly inverted on user input.")
    print("Press any key to invert state, q to exit:")

    subunit = 1
    bit = 1

    while True:

        userInput = input("Press to invert...")

        if(userInput):
            break

        # Get switch state
        bitState = card.ViewBit(subunit, bit)

        # Invert it
        bitState = not bitState

        # Set switch state
        card.OpBit(subunit, bit, bitState)




"""Example program for using the Pickering 41-743 Power Supply card with the Python pi743 ClientBridge wrapper."""

import pilxi
import pi743lx

if __name__ == "__main__":
    print("pi743lx wrapper version: {}".format(pi743lx.__version__))

    # IP address of the LXI
    IP_Address = "192.168.2.106"

    try:
        # Open a session with the LXI unit and get the session ID
        session = pilxi.Pi_Session(IP_Address)
        sessionID = session.GetSessionID()

    except pilxi.Error as ex:
        print("Error opening session with LXI: ", ex.message)
        exit()

    try:
        # Open the pi743lx library using the LXI session
        pi743Base = pi743lx.Base(sessionID)

        # List 41-743 cards present in LXI (as VISA-style resource strings)
        cards = pi743Base.findInstrumentsRsrc()

        for card in cards:
            print("Found 41-743 card at: ", card)

    except pi743lx.Error as ex:
        print("Error opening pi743lx library/listing cards:", ex.message)
        exit()

    try:
        # Open a card. With no arguments, pi743Base.openCard() will open the first card found
        # in the LXI chassis. Useful if there is only one present. Otherwise, a resource string
        # can be passed as an optional parameter:

        card = pi743Base.openCard()

        # Alternatively:
        #card = pi743Base.openCard(resource="myResource")

        # idQuery and reset can also be passed optionally:
        #card = pi743Base.openCard(resource="myResource", idQuery=True, reset=True)

    except pi743lx.Error as ex:
        print("Error opening 41-743 card: ", ex.message)
        exit()

    try:
        # Set connection status to disconnectedion
        connectionStatus = pi743Base.ConnectionStatus["CON_DISCONNECT"]
        card.setOutputConnection(connectionStatus)

        # Set a voltage (takes a float value in volts), and a current limit
        # (float value current in amps)
        voltage = 5.0
        current = 1.0

        card.setOutputCurrentLimit(current)
        card.setOutputVoltage(voltage)

        # Enable the DC-DC converter (enables output)
        card.setPowerSupplyStatus(True)

        connectionStatus = pi743Base.ConnectionStatus["CON_2WIRE_INTERNAL"]     # 2 wire internal sense
        #connectionStatus = pi743Base.ConnectionStatus["CON_4WIRE_EXTERNAL"]    # 4 wire external sense
        card.setOutputConnection(connectionStatus)

    except pi743lx.Error as ex:
        print("Error operating 41-743 card: ", ex.message)
        exit()

    card.close()
    session.Close()




from __future__ import print_function
import pilxi


base = pilxi.Pi_Base()

print("LXI Discovery example program using the Pickering ClientBridge Python wrapper")

print("pilxi wrapper version: {}".format(pilxi.__version__))

try:
    LXIs = base.Discover(address="192.168.0.255")
    print("Number of LXIs available:", len(LXIs))

# Print description of any errors and exit
except pilxi.Error as ex:
    print("Error occurred:", ex.message)
    exit()

for address, description in LXIs:
    print("Found LXI at {}, description: {}".format(address, description))




from setuptools import setup

setup(
    name='Pilxi',
    version='5.7',
    packages=['pilxi','pipslx', "pi620lx", "pi743lx", "LXIpxm78xx"],
    url='',
    license='',
    author='Pickering Interfaces',
    author_email='support@pickeringtest.com',
    description='Python wrapper Library for Pickering LXI ClientBridge driver',
)



from sys import version_info
import ctypes as c
import platform
import warnings
from enum import IntEnum

__version__ = "1.1.0"

# region Structs

class Complex_t(c.Structure):
    _fields_ = [
        ("magnitude", c.c_double),
        ("phase", c.c_double),
        ("realPart", c.c_double),
        ("imaginaryPart", c.c_double)
    ]


class DMM_SBRESULT(c.Structure):
    _fields_ = [
        ("frequency", c.c_double),
        ("dcOffset", c.c_double),
        ("acRms", c.c_double),
        ("acdcRms", c.c_double),
        ("sampleCount", c.c_uint32),
        ("nPeriods", c.c_uint32),
        ("sampleInterval", c.c_double),
        ("min", c.c_double),
        ("max", c.c_double),
        ("uncertainty", c.c_double),
        ("serialResistance", c.c_double),
        ("impedance", Complex_t),
        ("zValue", c.c_double),
        ("zPhase", c.c_double),
        ("uValue", c.c_double),
        ("uPhase", c.c_double),
        ("iValue", c.c_double),
        ("iPhase", c.c_double),
        ("nPerPeriod", c.c_double),
        ("dPhi", c.c_double),
        ("triggerDelay", c.c_double),
        ("meanCount", c.c_uint32)
    ]


class WFD_SAMPLECFG(c.Structure):
    _fields_ = [
        ("interval", c.c_double),
        ("preTrigger", c.c_uint32),
        ("postTrigger", c.c_uint32),
        ("triggerLevel", c.c_double),
        ("triggerHysteresis", c.c_double),
        ("triggerDelay", c.c_double),
        ("triggerSource", c.c_int),
        ("continueWithNext", c.c_uint16),
        ("noWaitForTrigger", c.c_uint16)
    ]


class WFD_SEGMENTCFG(c.Structure):
    _fields_ = [
        ("interval", c.c_uint32),              # sample rate devider
        ("preTrigger", c.c_uint32),            # number of samples before trigger event
        ("postTrigger", c.c_uint32),           # number of samples after trigger event
        ("triggerLevel", c.c_uint32),          # level of trigger comparator
        ("triggerHysteresis", c.c_uint32),     # hysteresis of trigger comparator
        ("triggerDelay", c.c_uint32),          # delay between trigger event and start of aquisition
        ("startAddr", c.c_uint32),             # start address of sampling data in instrument memory
        ("ctrlReg", c.c_uint32)                # control register (bit 7..0 -> Trigger source selector", c_uint32), bit 8 -> Continue / Stop)
    ]

# Trigger Matrix

class TM_INPUT_TAB(c.Structure):
    _fields_ = [
        ("inputMask", c.c_ulonglong),          # One entry of \ref PXM78xx_TM_CHANNEL_t in each element of the array. This value can be passed to any function expecting a TriggerMatrix channel as parameter.
        ("inputName", c.c_char * 64)           # Human readable description of the channel.
    ]

class TM_CHANNEL_TAB(c.Structure):
    _fields_ = [
        ("chanelIdx", c.c_int),                # One entry of \ref public_tm_input in each element of the array. This value can be passed to PXM78xx_TM_ConfigInputMask().
        ("channelName", c.c_char * 64)         # Human readable description of the input.
    ]

# endregion

# region Enums

class PXM78xx_STATUS(IntEnum):
    OK = 0,  # Instrument is operable
    STFAIL = 1,  # SelfTest failed
    CALINVALID = 2,  # calibration data invalid
    TEMPLOW = 3,  # Temperature too low
    TEMPHIGH = 4,  # Temperature too high
    AF_FAIL = 5,  # AnalogFrontend failure
    POWER_FAIL = 6,  # AnalogFrontend Power Supply failure
    DCDC_FAIL = 7,  # DCDC Converter failure
    PLL_FAIL = 8,  # PLL failure
    SDM_TIMEOUT = 9,  # SerDes Timeout
    DATA_MIGRATED = 10  # Functional limitation due to device data migration


class PXM78xx_ERRORCODES(IntEnum):
    FUNC_NOT_LOADED = -0x6000,  # Desired function not loaded from 'PXM78xx.dll' library.
    SW = -8000,  # unknown software error
    HW = -8001,  # unknown hardware error
    INV_PARAMETER = -8002,  # invalid parameter
    INV_HANDLE = -8003,  # invalid intrument handle
    NULLPOINTER = -8004,  # nullpointer exception
    INV_ADR = -8005,  # address out of range
    INV_VALUE = -8006,  # invalid input value
    INV_SAMPLERATE = -8007,  # invalid sample rate
    INV_CHANNEL = -8008,  # invalid channel
    INV_MODE = -8009,  # invalid mode
    INV_IMAGE = -8010,  # invalid update image
    INV_APERTURE = -8011,  # invalid aperture
    INV_HWCFG = -8012,  # invalid hardware configuration
    ID_QUERY = -8013,  # identification query failed
    I2C_ACCESS = -8014,  # I2C access failed
    I2C_NACK = -8015,  # no acknowlage from I2C slave
    TIMEOUT = -8016,  # communication timeout
    OPTION = -8017,  # option not available
    MEMORY = -8018,  # could not allocate memory
    FILE = -8019,  # opening or reading file failed
    CRC = -8020,  # invalid checksum
    VERIFY = -8021,  # verification of written data failed
    INV_CAL = -8023,  # calibration invalid
    NOT_INIT = -8024,  # instrument not initialized
    DATAID = -8025,  # incompatible calibration data
    NO_CAL_VAL = -8026,  # no calibration available
    NO_RANGE = -8027,  # no suitable range available
    NO_FILTER = -8028,  # no suitable filter available
    WAITTRG = -8029,  # waiting for trigger
    SEGMENT_INACTIVE = -8030,  # segment is inactive
    ANALOG_BOARD = -8031,  # analog board not available
    NOT_SUPPORTED = -8032,  # feature or function not supported
    INV_FUNC = -8033,  # invalid measurement function
    INV_ADJVAL = -8034,  # adjustment value is not acceptable
    INV_FIRMWARE = -8035,  # instrument firmware is not compatible
    RSC_INACTIVE = -8036,  # resource is inactive
    TRIGGER_FAILURE = -8037,  # general trigger failure
    INV_VERSION = -8038,  # a given version is not compatible
    INV_BUF_SIZE = -8039,  # a given buffer size is not sufficient
    # Warnings
    WARN_CORRUPT_DATA = 5000,  # data in NV-memory is corrupt
    WARN_INC_APERTURE = 5001,  # increase aperture
    WARN_DEC_APERTURE = 5002,  # decrease aperture
    WARN_NO_SIGNAL = 5003,  # no input signal
    WARN_IRNORED = 5004,  # setting is ignored in current condition.
    WARN_OVRNG = 5005,  # input signal is out of acceptable range.


class PXM78xx_RANGE:
    DCV_10mV = 0.010  # 10mV  DC voltage range
    DCV_100mV = 0.100  # 1000mV DC voltage range
    DCV_125mV = 0.125  # 250mV DC voltage range
    DCV_250mV = 0.25  # 250mV DC voltage range
    DCV_500mV = 0.50  # 500mV DC voltage range
    DCV_1V = 1.00  # 1V    DC voltage range
    DCV_2V = 2.00  # 2V    DC voltage range
    DCV_4V = 4.00  # 4V    DC voltage range
    DCV_8V = 8.00  # 8V    DC voltage range
    DCV_16V = 16.00  # 16V   DC voltage range
    DCV_32V = 32.00  # 32V   DC voltage range
    DCV_64V = 64.00  # 64V   DC voltage range
    DCV_128V = 128.00  # 128V  DC voltage range
    DCV_256V = 256.00  # 256V  DC voltage range

    ACV_5mV = 0.005  # 5mV   AC voltage range
    ACV_50mV = 0.050  # 50mV  AC voltage range
    ACV_125mV = 0.125  # 125mV AC voltage range
    ACV_250mV = 0.25  # 250mV AC voltage range
    ACV_500mV = 0.50  # 500mV AC voltage range
    ACV_1V = 1.00  # 1V    AC voltage range
    ACV_2V = 2.00  # 2V    AC voltage range
    ACV_4V = 4.00  # 4V    AC voltage range
    ACV_8V = 8.00  # 8V    AC voltage range
    ACV_16V = 16.00  # 16V   AC voltage range
    ACV_32V = 32.00  # 32V   AC voltage range
    ACV_64V = 64.00  # 64V   AC voltage range
    ACV_128V = 128.00  # 128V  AC voltage range

    DCA_10mA = 0.010  # 10mA  DC current range
    DCA_100mA = 0.100  # 100mA DC current range
    DCA_1A = 1.000  # 1A    DC current range

    ACA_10mA = 0.010  # 10mA  AC current range
    ACA_100mA = 0.100  # 100mA AC current range
    ACA_1A = 1.000  # 1A    AC current range

    L_10uH = 1.00e-5  # 10uH  inductance range
    L_100uH = 1.00e-4  # 100uH inductance range
    L_1mH = 1.00e-3  # 1mH   inductance range
    L_10mH = 1.00e-2  # 10mH  inductance range
    L_100mH = 1.00e-1  # 100mH inductance range
    L_1H = 1.00e-0  # 1H    inductance range

    C_1nF = 1.00e-9  # 1nF   capacitance range
    C_10nF = 1.00e-8  # 10nF  capacitance range
    C_100nF = 1.00e-7  # 100nF capacitance range
    C_1uF = 1.00e-6  # 1uF   capacitance range
    C_10uF = 1.00e-5  # 10uF  capacitance range
    C_100uF = 1.00e-4  # 100uF capacitance range
    C_1mF = 1.00e-3  # 1mF   capacitance range
    C_10mF = 1.00e-2  # 10mF  capacitance range

    R_100OHM = 1.00e+2  # 100Ohm  resistance range
    R_1kOHM = 1.00e+3  # 1kOhm   resistance range
    R_10kOHM = 1.00e+4  # 10kOhm  resistance range
    R_100kOHM = 1.00e+5  # 100kOhm resistance range
    R_1MOHM = 1.00e+6  # 1MOhm   resistance range
    R_10MOHM = 1.00e+7  # 10MOhm  resistance range


class PXM78xx_FUNC(IntEnum):
    DMM_DCV = 0,  # DC voltage measurement
    DMM_ACV = 1,  # AC voltage measurement with AC coupling
    DMM_ACDCV = 2,  # AC+DC voltage measurement with DC coupling. Result is RMS of applied signal (AC+DC part).
    DMM_DCC = 3,  # DC current measurement
    DMM_ACC = 4,  # AC current measurement
    DMM_L2W = 5,  # inductance measurement 2 wire (not available on all models)
    DMM_L4W = 6,  # inductance measurement 4 wire (not available on all models)
    DMM_C2W = 7,  # capacitance measurement 2 wire (not available on all models)
    DMM_C4W = 8,  # capacitance measurement 4 wire (not available on all models)
    DMM_R2W = 9,  # resistance measurement 2 wire
    DMM_R4W = 10,  # resistance measurement 4 wire
    DMM_DIODE = 11,  # diode measurement
    WFD_DCV = 12,  # waveform digitizer voltage
    WFD_DCC = 13,  # waveform digitizer current
    DMM_ACV_DC_COUPLED = 14,  # AC voltage measurement with DC Coupling. Result is RMS of AC part.


class PXM78xx_FILTER(IntEnum):
    FILTER_100Hz = 100,  # 100 Hz filter.
    FILTER_1kHz = 1000,  # 1 kHz filter.
    FILTER_10kHz = 10000,  # 10 kHz filter.
    FILTER_100kHz = 100000,  # 100 kHz filter.
    FILTER_1MHz = 1000000,  # 1 MHz filter.
    FILTER_NONE = 9999999,  # no filter for maximum analog bandwidth.
    FILTER_INVALID = -1  # invalid filter setting.


class PXM78xx_CHANNEL(IntEnum):
    ADC16BIT = 0,  # 16 Bit 20 MSps ADC
    ADC24BIT = 1  # 24 Bit 1 MSps ADC (not available on all models)


class PXM78xx_TM_EDGE(IntEnum):
    FALLING = 0,  # trigger on falling edge of the input signal.
    RISING = 1  # trigger on rising edge of the input signal.


class PXM78xx_CABLECOMPENSATION_MODE(IntEnum):
    R2W = 0,  # compensate to zero, using R2W measurement only.
    R4W = 1  # compensate to R4W result, using R2W and R4W measurment.


class PXM78xx_WFD_STATUS(IntEnum):
    PXM78xx_WFD_STATUS_IDLE = 0,  # Sample Engine is stopped. Call \ref PXM78xx_WFD_StartSampling to start the Sampling Engine.
    PXM78xx_WFD_STATUS_WAITING = 1,  # Pre-trigger samples have been acquired and the Sample Engine is waiting for a trigger event to start acquisition of post-trigger samples.
    PXM78xx_WFD_STATUS_TRIGGERED = 2,  # Sampling Engine is triggered an will begin with data acquisition in the next clock cycle.
    PXM78xx_WFD_STATUS_FINISHED = 3,  # Acquisition is finished. Data can be fetched.
    PXM78xx_WFD_STATUS_ERROR = 4,  # FIFO has run full. If this happens something went fundamentally wrong.
    PXM78xx_WFD_STATUS_RUNNING = 5,  # Data acquisition in progress


class PXM78xx_COUPLING(IntEnum):
    PXM78xx_COUPLING_DC = 0,
    PXM78xx_COUPLING_AC = 1


class PXM78xx_WFD_TRIGGER(IntEnum):
    EXTERN  = (1<<7),               # input from trigger matrix
    RISING  = (1<<7)|(1<<0),        # measurement signal rising edge
    FALLING = (1<<7)|(1<<1),        # measurement signal falling edge
    BOTH    = (1<<7)|(1<<0)|(1<<1)  # measurement signal rising or falling edge

# Trigger Matrix definitions

class PXM78xx_TM_INPUT(IntEnum):
    SOFTWARE        = (0x0000000000000800 <<  0)  # PXI trigger line 0
    PXI_TRIG0       = (0x0000000000001000 <<  0)  # PXI trigger line 0
    PXI_TRIG1       = (0x0000000000001000 <<  1)  # PXI trigger line 1
    PXI_TRIG2       = (0x0000000000001000 <<  2)  # PXI trigger line 2
    PXI_TRIG3       = (0x0000000000001000 <<  3)  # PXI trigger line 3
    PXI_TRIG4       = (0x0000000000001000 <<  4)  # PXI trigger line 4
    PXI_TRIG5       = (0x0000000000001000 <<  5)  # PXI trigger line 5
    PXI_TRIG6       = (0x0000000000001000 <<  6)  # PXI trigger line 6
    PXI_TRIG7       = (0x0000000000001000 <<  7)  # PXI trigger line 7
    PXI_STARTRIG    = (0x0000000000001000 <<  9)  # PXI star trigger

    ADC16_CMP1_RISE = (0x0000000000001000 << 11)  # 16Bit ADC Comparator 1 Rising Edge
    ADC16_CMP1_FALL = (0x0000000000001000 << 12)  # 16Bit ADC Comparator 1 Falling Edge
    ADC16_CMP2_RISE = (0x0000000000001000 << 13)  # 16Bit ADC Comparator 2 Rising Edge
    ADC16_CMP2_FALL = (0x0000000000001000 << 14)  # 16Bit ADC Comparator 2 Falling Edge
    ADC16_CMP3_RISE = (0x0000000000001000 << 15)  # 16Bit ADC Comparator 3 Rising Edge
    ADC16_CMP3_FALL = (0x0000000000001000 << 23)  # 16Bit ADC Comparator 3 Falling Edge
    ADC16_SMPL      = (0x0000000000001000 << 24)  # 16Bit ADC Sampling Trigger

    ADC24_CMP1_RISE = (0x0000000000001000 << 16)  # 24Bit ADC Comparator 1 Rising Edge
    ADC24_CMP1_FALL = (0x0000000000001000 << 17)  # 24Bit ADC Comparator 1 Falling Edge
    ADC24_CMP2_RISE = (0x0000000000001000 << 18)  # 24Bit ADC Comparator 2 Rising Edge
    ADC24_CMP2_FALL = (0x0000000000001000 << 19)  # 24Bit ADC Comparator 2 Falling Edge
    ADC24_SMPL      = (0x0000000000001000 << 20)  # 24Bit ADC Sampling Trigger

    FRONT2          = (0x0000000000001000 << 21)  # external front panel input connector
    FRONT1          = (0x0000000000001000 << 22)  # external front panel input connector
    FLOAT           = (0x0000000000001000 <<  0)  # trigger line from floating FPGA
    
class PXM78xx_TM_CHANNEL(IntEnum):
    PXI0     = 0,  # PXI trigger line 0
    PXI1     = 1,  # PXI trigger line 1
    PXI2     = 2,  # PXI trigger line 2
    PXI3     = 3,  # PXI trigger line 3
    PXI4     = 4,  # PXI trigger line 4
    PXI5     = 5,  # PXI trigger line 5
    PXI6     = 6,  # PXI trigger line 6
    PXI7     = 7,  # PXI trigger line 7
    WFD16    = 8,  # 16Bit ADC Sample Engine external trigger
    WFD24    = 13, # 24Bit ADC Sample Engine external trigger
    ADC16    = 9,  # 16Bit ADC trigger
    ADC24    = 10, # 24Bit ADC trigger
    FRONT1   = 15, # Front Trigger 1
    FRONT2   = 14, # Front Trigger 2
    TC1_0    = 11, # Timer/Counter Channel 1 Input 0
    TC1_1    = 12, # Timer/Counter Channel 1 Input 1
    TC2_0    = 16, # Timer/Counter Channel 2 Input 0
    TC2_1    = 17  # Timer/Counter Channel 2 Input 1

class PXM78xx_TM_INPUT_FUNC(IntEnum):
    OR  = 0,     # require one of the specified trigger inputs (logical OR)
    AND = 1      # require all of the specified trigger inputs (logical AND)

class PXM78xx_TM_EDGE(IntEnum):
    FALLING = 0, # trigger on falling edge of the input signal.
    RISING  = 1  # trigger on rising edge of the input signal.

class PXM78xx_TM_MODE(IntEnum):
    ASYNC        = 0, # Asynchronous trigger routing optimized for minimal trigger latency.
    SYNC_LEVEL   = 1, # Synchronous trigger routing. Input signals are synchonized to the TriggerMatrix Clock. This Mode is prefered for external trigger signals (PXI Backplane or Front) which are routed to an internal trigger reciever (Timer/Counter, Waveform Digitizer, ...).
    SYNC_SLOPE   = 2  # Synchronous trigger routung with slope generation. This mode features edge detection, single shot, configurable pulse width and delay.

# Timer/Counter definitions

# Timer/Counter Channel
class PXM78xx_TC_CHANNEL(IntEnum):
    TC1 = 0,                      # TimerCounter Channel 1
    TC2 = 1                       # TimerCounter Channel 2

# Timer/Counter Status
class PXM78xx_TC_STATUS(IntEnum):
    BUSY      = 0x0,              # Timer/Counter is busy doing very important shit and things
    FINISHED  = 0x1,              # Timer/Counter has finished and measurement result is ready
    WAITTRG   = 0x2,              # Timer/Counter is waiting for a trigger event
    IDLE      = 0x4               # Timer/Counter is ready to start

class PXM78xx_TC_MODE(IntEnum):
    TIMER_AA          = 0x01,     # measure time between two events on Trigger A.
    TIMER_AB          = 0x02,     # measure time between an event on Trigger A and Trigger B.
    COUNTER_GATE      = 0x04,     # count events of TriggerA during GateTime
    COUNTER_B         = 0x08,     # count events of TriggerA while Trigger B is HIGH
    TIMER_ABA         = 0x10      # measure time between an event of Trigger A and Trigger B (result1) and two events on Trigger A (result2). This mode is useful for PulseWidthRatio measurement if Trigger A is rising edge and Trigger B is falling edge of one signal.


# Timer/Counter Trigger Input
# These values can combined with an bitwise OR (|) if necessary.
# PXM78xx_TC_ConfigTriggerInput()

class PXM78xx_TC_TRGINPUT(IntEnum):
    NONE          = 0x00,         # leave the Timer/Counter trigger unconnected
    IN0_RISING    = 0x01,         # connect the Timer/Counter trigger to input 0 of the TriggerMatrix with detection of the rising edge. 
    IN0_FALLING   = 0x02,         # connect the Timer/Counter trigger to input 0 of the TriggerMatrix with detection of the falling edge.
    IN0_LEVEL     = 0x04,         # connect the Timer/Counter trigger to input 0 of the TriggerMatrix without edge detection.
    IN1_RISING    = 0x08,         # connect the Timer/Counter trigger to input 1 of the TriggerMatrix with detection of the rising edge.
    IN1_FALLING   = 0x10,         # connect the Timer/Counter trigger to input 1 of the TriggerMatrix with detection of the falling edge.
    IN1_LEVEL     = 0x20,         # connect the Timer/Counter trigger to input 1 of the TriggerMatrix without edge detection.
    MASK          = 0xFF

# endregion


class PXM78xx_Error(Exception):
    def __init__(self, message, errorCode=None):
        self.message = message
        self.errorCode = errorCode

    def __str__(self):
        return self.message


class PXM78xx_Base:

    def __init__(self, session):
        self.session = session
        self.vi = 0

        if platform.system() == "Windows":
            arch = platform.architecture()
            if "64bit" in arch:
                self._handle = c.windll.LoadLibrary("LXI_PXM78xx_w64")

            else:
                self._handle = c.windll.LoadLibrary("LXI_PXM78xx_w32")

        self.pythonMajorVersion = version_info[0]

    #region Private methods for error handling and Python 2/3 string issues

    def _stringToStr(self, inputString):
        """Take a string passed to a function in Python 2 or Python 3 and convert to
           a ctypes-friendly ASCII string"""

        # Check for Python 2 or 3
        if self.pythonMajorVersion < 3:
            if type(inputString) is str:
                return inputString
            if type(inputString) is unicode:
                return inputString.encode()
        else:
            if type(inputString) is bytes:
                return inputString
            elif type(inputString) is str:
                return inputString.encode()

    def _pythonString(self, inputString):
        """Ensure returned strings are native in Python 2 and Python 3"""

        # Check for Python 2 or 3
        if self.pythonMajorVersion < 3:
            try:
                return inputString.value
            except AttributeError:
                return inputString
        else:
            try:
                return inputString.value.decode()
            except AttributeError:
                return inputString.decode()

    def _handleError(self, error):

        if error:
            errorString = c.create_string_buffer(256)
            self._handle.LXI_PXM78xx_error_message(self.session, 0, int(error), c.byref(errorString), 256)
            errorString = self._pythonString(errorString.value)
            raise PXM78xx_Error(errorString, errorCode=error)
        return

    #endregion

    #region Discovery and connection functions

    def getVersion(self):

        bufferSize = 256
        rev = c.create_string_buffer(bufferSize)

        err = self._handle.LXI_PXM78xx_GetVersion(rev, bufferSize)
        self._handleError(err)

        return self._pythonString(rev.value)

    def QueryLibraryVersions(self):

        bufferSize = 256
        rev = c.create_string_buffer(bufferSize)

        err = self._handle.LXI_PXM78xx_QueryLibraryVersions(self.session, rev, bufferSize)
        self._handleError(err)

        return "PyWrap_LXI_PXM78xx: " + __version__ + "\n" + self._pythonString(rev.value)

    def findFreeInstruments(self):

        bufferSize = 256
        resourceNames = ((c.c_char * bufferSize) * 10)()
        resourceCount = c.c_uint32(10)
        buslist = (c.c_uint32 * bufferSize)()
        slotlist = (c.c_uint32 * bufferSize)()

        err = self._handle.LXI_PXM78xx_FindFreeInstruments(self.session, resourceNames, c.byref(resourceCount), buslist, slotlist)
        self._handleError(err)

        resources = [self._pythonString(name.value) for name in resourceNames if self._pythonString(name.value) != ""]

        return resources

    def autoConnectToFirst(self):

        instrHandle = c.c_uint32()

        err = self._handle.LXI_PXM78xx_autoConnectToFirst(self.session, c.byref(instrHandle))
        self._handleError(err)

        instrument = _PXM78xx_Card(self.session, instrHandle)

        return instrument

    def autoConnectToAll(self):

        instrArrayLength = c.c_uint16(16)
        instrArray = (c.c_uint32 * instrArrayLength.value)()
        numConnected = c.c_uint32(0)

        err = self._handle.LXI_PXM78xx_autoConnectToAll(self.session,
                                                        instrArray,
                                                        instrArrayLength,
                                                        c.byref(numConnected))
        self._handleError(err)

        instruments = []

        for i in range(numConnected.value):
            instruments.append(_PXM78xx_Card(self.session, instrArray[i]))

        return instruments


    #endregion

class _PXM78xx_Card(PXM78xx_Base):

    #region Constructor, error and destructor methods

    def __init__(self, session, vi):

        PXM78xx_Base.__init__(self, session)

        self.vi = vi
        self.session = session
        self.disposed = False

    def _close(self):
        if not self.disposed:
            self._handle.LXI_PXM78xx_close(self.session, self.vi)
            self.disposed = True
        return

    def __del__(self):
        if not self.disposed:
            self._close()
        return

    def _handleError(self, error):

        if error:
            errorString = c.create_string_buffer(256)
            self._handle.LXI_PXM78xx_error_message(self.session, self.vi, int(error), c.byref(errorString), 256)
            errorString = self._pythonString(errorString.value)

            # If the error code is a warning (between 5000 and 5005)
            if 5000 <= error <= 5005:
                warnings.warn(errorString)

            # Else it's just a regular error
            else:
                raise PXM78xx_Error(errorString, errorCode=error)
        return

    #endregion

    #region Self test, reset, and instrument identification methods

    def reset(self):

        err = self._handle.LXI_PXM78xx_reset(self.session, self.vi)
        self._handleError(err)

        return

    def selfTest(self):

        testResult = c.c_uint16()
        messageBufferLength = c.c_uint32(256)
        testMessage = c.create_string_buffer(messageBufferLength.value)

        err = self._handle.LXI_PXM78xx_self_test(self.session,
                                                 self.vi,
                                                 c.byref(testResult),
                                                 testMessage,
                                                 messageBufferLength)
        self._handleError(err)

        return testResult.value, self._pythonString(testMessage.value)

    def resetFuse(self):

        err = self._handle.LXI_PXM78xx_AF_ResetFuse(self.session, self.vi)
        self._handleError(err)

        return

    def queryStatus(self):

        status = c.c_int()

        err = self._handle.LXI_PXM78xx_QueryStatus(self.session, self.vi, c.byref(status))
        self._handleError(err)

        return status.value

    def queryStatusMessage(self):

        message = c.create_string_buffer(64)
        messageLen = c.c_uint32(64)

        err = self._handle.LXI_PXM78xx_QueryStatusMessage(self.session, self.vi, message, messageLen)
        self._handleError(err)

        return self._pythonString(message)

    def queryTemperature(self):

        tempShunt = c.c_double(0)
        tempFPGA  = c.c_double(0)

        err = self._handle.LXI_PXM78xx_QueryTemperature(self.session, self.vi, c.byref(tempFPGA), c.byref(tempShunt))
        self._handleError(err)

        return tempFPGA.value, tempShunt.value

    def querySlotNumber(self):

        slot = c.c_int16(0)

        err = self._handle.LXI_PXM78xx_QuerySlotNumber(self.session, self.vi, c.byref(slot))
        self._handleError(err)

        return slot.value

    def queryInstrumentLocation(self):

        bus = c.c_uint32(0)
        device = c.c_uint32(0)

        err = self._handle.LXI_PXM78xx_QueryInstrumentLocation(self.session, self.vi, c.byref(bus), c.byref(device))
        self._handleError(err)

        return bus.value, device.value

    def querySerialNumber(self):

        serialNumber = c.create_string_buffer(10)
        serialNumberLen = c.c_uint32(10)

        err = self._handle.LXI_PXM78xx_QuerySerialNumber(self.session, self.vi, serialNumber, serialNumberLen)
        self._handleError(err)

        return self._pythonString(serialNumber)

    def queryProductName(self):

        productName = c.create_string_buffer(128)
        productNameLen = c.c_uint32(128)

        err = self._handle.LXI_PXM78xx_QueryProductName(self.session, self.vi, productName, productNameLen)
        self._handleError(err)

        return self._pythonString(productName)

    #endregion

    #region DMM methods

    def DMM_configFunction(self, function):

        function = c.c_int(function)

        err = self._handle.LXI_PXM78xx_DMM_ConfigFunction(self.session, self.vi, function)
        self._handleError(err)

        return

    def DMM_configRange(self, measureRange):

        measureRange = c.c_double(measureRange)

        err = self._handle.LXI_PXM78xx_DMM_ConfigRange(self.session, self.vi, measureRange)
        self._handleError(err)

        return

    def DMM_configMeasurement(self, function, measureRange):

        function = c.c_int(function)
        measureRange = c.c_double(measureRange)

        err = self._handle.LXI_PXM78xx_DMM_ConfigMeasurement(self.session, self.vi, function, measureRange)
        self._handleError(err)

        return

    def DMM_queryMeasurement(self):

        function = c.c_int(0)
        measureRange = c.c_double(0.0)

        err = self._handle.LXI_PXM78xx_DMM_QueryMeasurement(self.session,
                                                            self.vi,
                                                            c.byref(function),
                                                            c.byref(measureRange))
        self._handleError(err)

        return function.value, measureRange.value

    def DMM_configFilter(self, bwFilter):

        bwFilter = c.c_int(bwFilter)

        err = self._handle.LXI_PXM78xx_DMM_ConfigFilter(self.session, self.vi, bwFilter)
        self._handleError(err)

        return

    def DMM_queryFilter(self):

        bwFilter = c.c_int(0)

        err = self._handle.LXI_PXM78xx_DMM_QueryFilter(self.session, self.vi, c.byref(bwFilter))
        self._handleError(err)

        return bwFilter.value

    def DMM_forceTrigger(self):

        err = self._handle.LXI_PXM78xx_DMM_ForceTrigger(self.session. self.vi)
        self._handleError(err)

        return

    def DMM_singleMeasure(self):

        result = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_SingleMeasure(self.session, self.vi, c.byref(result))
        self._handleError(err)

        return result.value

    def DMM_meanMeasure(self, aperture):

        aperture = c.c_double(aperture)
        result = c.c_double()
        sbResult = DMM_SBRESULT()

        err = self._handle.LXI_PXM78xx_DMM_MeanMeasure(self.session, self.vi, aperture, c.byref(result), c.byref(sbResult))
        self._handleError(err)

        return result.value, sbResult

    def DMM_fetchSingleResult(self):

        result = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_FetchSingleResult(self.session, self.vi, c.byref(result))
        self._handleError(err)

        return result.value

    def DMM_fetchMeanResult(self):

        result = c.c_double()
        sbResult = DMM_SBRESULT()

        err = self._handle.LXI_PXM78xx_DMM_FetchMeanResult(self.session, self.vi, c.byref(result), c.byref(sbResult))
        self._handleError(err)

        return result.value, sbResult

    def DMM_queryAccuracy(self, signalFrequency, result):

        result = c.c_double(result)
        signalFrequency = c.c_double(signalFrequency)
        uncertainty = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_QueryAccuracy(self.session,
                                                         self.vi,
                                                         signalFrequency,
                                                         result,
                                                         c.byref(uncertainty))
        self._handleError(err)

        return uncertainty.value

    def DMM_configAperture(self, aperture):

        aperture = c.c_double(aperture)
        sbResult = DMM_SBRESULT()

        err = self._handle.LXI_PXM78xx_DMM_ConfigAperture(self.session, self.vi, aperture, c.byref(sbResult))
        self._handleError(err)

        return sbResult

    def DMM_queryAperture(self):

        aperture = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_QueryAperture(self.session, self.vi, c.byref(aperture))
        self._handleError(err)

        return aperture.value

    def DMM_init(self):

        err = self._handle.LXI_PXM78xx_DMM_Init(self.session, self.vi)
        self._handleError(err)

        return

    def DMM_queryFilterList(self):

        filterValuesLen = c.c_uint32(5)
        filterValues = (c.c_uint32 * filterValuesLen.value)()
        filterLabelsLen = c.c_uint32(40)
        filterLabels = c.create_string_buffer(filterLabelsLen.value)

        err = self._handle.LXI_PXM78xx_DMM_QueryFilterList(self.session,
                                                           self.vi,
                                                           c.byref(filterValues),
                                                           c.byref(filterValuesLen),
                                                           c.byref(filterLabels),
                                                           filterLabelsLen)
        self._handleError(err)

        filterValuesList = []
        for i in range(filterValuesLen.value):
            filterValuesList.append(filterValues[i])

        filterLabelsList = self._pythonString(filterLabels.value).split(";")

        filtersDict = {filterLabelsList[i]: filterValuesList[i] for i in range(len(filterValuesList))}

        return filtersDict

    def DMM_queryChannel(self):

        channel = c.c_int()

        err = self._handle.LXI_PXM78xx_DMM_QueryChannel(self.session, self.vi, c.byref(channel))
        self._handleError(err)

        return channel.value

    def DMM_queryChannelByFunction(self, function, measureRange):

        function = c.c_int(function)
        measureRange = c.c_double(measureRange)
        channel = c.c_int()

        err = self._handle.LXI_PXM78xx_DMM_QueryChannelByFunction(self.session,
                                                                  self.vi,
                                                                  c.byref(channel),
                                                                  function,
                                                                  measureRange)
        self._handleError(err)

        return function.value

    def DMM_queryTriggerChannel(self):

        tmChannel = c.c_int()

        err = self._handle.LXI_PXM78xx_DMM_QueryTriggerChannel(self.session, self.vi, c.byref(tmChannel))
        self._handleError(err)

        return tmChannel.value

    def DMM_configTrigger(self, triggerSource, triggerDelay, holdOff, edge):

        triggerSource = c.c_uint64(triggerSource)
        triggerDelay = c.c_double(triggerDelay)
        holdOff = c.c_double(holdOff)
        edge = c.c_int(edge)

        err = self._handle.LXI_PXM78xx_DMM_ConfigTrigger(self.session,
                                                         self.vi,
                                                         triggerSource,
                                                         triggerDelay,
                                                         holdOff,
                                                         edge)
        self._handleError(err)

        return

    def DMM_queryTrigger(self):

        triggerSource = c.c_uint64()
        triggerDelay = c.c_double()
        holdOff = c.c_double()
        edge = c.c_int()

        err = self._handle.LXI_PXM78xx_DMM_QueryTrigger(self.session,
                                                        self.vi,
                                                        c.byref(triggerSource),
                                                        c.byref(triggerDelay),
                                                        c.byref(holdOff),
                                                        c.byref(edge))
        self._handleError(err)

        return triggerSource.value, triggerDelay.value, holdOff.value, edge.value

    def DMM_calculateElementsFromImpedance(self, parallel):

        parallel = c.c_uint32(parallel)
        sbResult = DMM_SBRESULT()
        resistance = c.c_double()
        capacitance = c.c_double()
        inductance = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_CalculateElementsFromImpedance(self.session,
                                                                          self.vi,
                                                                          parallel,
                                                                          c.byref(sbResult),
                                                                          c.byref(resistance),
                                                                          c.byref(capacitance),
                                                                          c.byref(inductance))
        self._handleError(err)

        return sbResult, resistance.value, capacitance.value, inductance.value

    def DMM_waitUntilReadyForTrigger(self):

        err = self._handle.LXI_PXM78xx_DMM_WaitUntilReadyForTrigger(self.session, self.vi)
        self._handleError(err)

        return

    def DMM_waitUntilFinished(self):

        err = self._handle.LXI_PXM78xx_DMM_WaitUntilFinished(self.session, self.vi)
        self._handleError(err)

        return

    def DMM_isOverRange(self, value):

        value = c.c_double(value)

        err = self._handle.LXI_PXM78xx_DMM_IsOverRange(self.session, value)

        if err == PXM78xx_ERRORCODES.WARN_OVRNG:
            return True
        else:
            return False

    def DMM_configACFrequencyMin(self, value):

        value = c.c_double(value)

        err = self._handle.LXI_PXM78xx_DMM_ConfigACFrequencyMin(self.session, self.vi, value)
        self._handleError(err)

        return

    def DMM_configACFrequencyMax(self, value):

        value = c.c_double(value)

        err = self._handle.LXI_PXM78xx_DMM_ConfigACFrequencyMax(self.session, self.vi, value)
        self._handleError(err)

        return

    def DMM_queryACFrequencyMin(self):

        value = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_QueryACFrequencyMin(self.session, self.vi, c.byref(value))
        self._handleError(err)

        return value.value

    def DMM_queryACFrequencyMax(self):

        value = c.c_double()

        err = self._handle.LXI_PXM78xx_DMM_QueryACFrequencyMax(self.session, self.vi, c.byref(value))
        self._handleError(err)

        return value.value

    def DMM_R2W_cableCompensation(self, mode, measureRange):

        mode = c.c_int(mode)
        measureRange = c.c_double(measureRange)

        err = self._handle.LXI_PXM78xx_DMM_R2W_CableCompensation(self.session,
                                                                 self.vi,
                                                                 mode,
                                                                 measureRange)
        self._handleError(err)

        return

    #endregion

    #region Waveform Digitiser methods

    def WFD_configSampling(self, sampleCfg):

        err = self._handle.LXI_PXM78xx_WFD_ConfigSampling(self.session, self.vi, c.byref(sampleCfg))
        self._handleError(err)

        return

    def WFD_querySampling(self):

        sampleCfg = WFD_SAMPLECFG()

        err = self._handle.LXI_PXM78xx_WFD_QuerySampling(self.session, self.vi, c.byref(sampleCfg))
        self._handleError(err)

        return sampleCfg

    def WFD_queryStatus(self):

        status = c.c_int()

        # last parameter reserved for future use
        err = self._handle.LXI_PXM78xx_WFD_QueryStatus(self.session, self.vi, c.byref(status), c.c_uint32(0))
        self._handleError(err)

        return status.value

    def WFD_queryStatusMessage(self):

        messageLen = c.c_uint32(64)
        message = c.create_string_buffer(messageLen.value)

        err = self._handle.LXI_PXM78xx_WFD_QueryStatusMessage(self.session, self.vi, c.byref(message), messageLen)
        self._handleError(err)

        return self._pythonString(message.value)

    def WFD_querySampleData(self, sampleCount):

        yData = (c.c_double * sampleCount)()
        xData = (c.c_double * sampleCount)()
        sampleCount = c.c_uint32(sampleCount)

        err = self._handle.LXI_PXM78xx_WFD_QuerySampleData(self.session,
                                                           self.vi,
                                                           c.byref(yData),
                                                           c.byref(xData),
                                                           c.byref(sampleCount))
        self._handleError(err)

        x = []
        y = []

        for i in range(sampleCount.value):
            y.append(yData[i])
            x.append(xData[i])

        return x, y

    def WFD_startSampling(self):

        err = self._handle.LXI_PXM78xx_WFD_StartSampling(self.session, self.vi)
        self._handleError(err)

        return

    def WFD_stopSampling(self):

        err = self._handle.LXI_PXM78xx_WFD_StopSampling(self.session, self.vi)
        self._handleError(err)

        return

    def WFD_init(self, channel):

        channel = c.c_int(channel)

        err = self._handle.LXI_PXM78xx_WFD_Init(self.session, self.vi, channel)
        self._handleError(err)

        return

    def WFD_queryTriggerChannel(self):

        channel = c.c_int()

        err = self._handle.LXI_PXM78xx_WFD_QueryTriggerChannel(self.session, self.vi, c.byref(channel))
        self._handleError(err)

        return channel.value

    def WFD_configMeasurement(self, function, measureRange):

        function = c.c_int(function)
        measureRange = c.c_double(measureRange)

        err = self._handle.LXI_PXM78xx_WFD_ConfigMeasurement(self.session, self.vi, function, measureRange)
        self._handleError(err)

        return

    def WFD_queryMeasurement(self):

        function = c.c_int()
        measureRange = c.c_double()

        err = self._handle.LXI_PXM78xx_WFD_ConfigMeasurement(self.session,
                                                             self.vi,
                                                             c.byref(function),
                                                             c.byref(measureRange))
        self._handleError(err)

        return function.value, measureRange.value

    def WFD_configSegment(self, segment, sampleCfg):

        segment = c.c_uint32(segment)

        err = self._handle.LXI_PXM78xx_WFD_ConfigSegment(self.session,
                                                         self.vi,
                                                         segment,
                                                         c.byref(sampleCfg))
        self._handleError(err)

        return

    def WFD_querySegment(self, segment):

        segment = c.c_uint32(segment)
        sampleCfg = WFD_SAMPLECFG()
        startAddr = c.c_uint32()

        err = self._handle.LXI_PXM78xx_WFD_QuerySegment(self.session,
                                                        self.vi,
                                                        segment,
                                                        c.byref(sampleCfg),
                                                        c.byref(startAddr))
        self._handleError(err)

        return sampleCfg, startAddr.value

    def WFD_querySegmentData(self, segment, sampleCount):

        xData = (c.c_double * sampleCount)()
        yData = (c.c_double * sampleCount)()
        sampleCount = c.c_uint32(sampleCount)
        segment = c.c_uint32(segment)
        segmentGap = c.c_double()

        err = self._handle.LXI_PXM78xx_WFD_QuerySegmentData(self.session,
                                                            self.vi,
                                                            segment,
                                                            c.byref(yData),
                                                            c.byref(xData),
                                                            c.byref(sampleCount),
                                                            c.byref(segmentGap))
        self._handleError(err)

        x = []
        y = []

        for i in range(sampleCount.value):
            y.append(yData[i])
            x.append(xData[i])

        return x, y, segmentGap.value

    def WFD_configComparator(self, compChannel, comparatorLevel, comparatorHysteresis):

        compChannel = c.c_uint32(compChannel)
        comparatorLevel = c.c_double(comparatorLevel)
        comparatorHysteresis = c.c_double(comparatorHysteresis)

        err = self._handle.LXI_PXM78xx_WFD_ConfigComparator(self.session,
                                                            self.vi,
                                                            compChannel,
                                                            comparatorLevel,
                                                            comparatorHysteresis)
        self._handleError(err)

        return

    def WFD_queryComparator(self, compChannel):

        compChannel = c.c_uint32(compChannel)
        comparatorLevel = c.c_double()
        comparatorHysteresis = c.c_double()

        err = self._handle.LXI_PXM78xx_WFD_QueryComparator(self.session,
                                                           self.vi,
                                                           compChannel,
                                                           c.byref(comparatorLevel),
                                                           c.byref(comparatorHysteresis))
        self._handleError(err)

        return comparatorLevel.value, comparatorHysteresis.value

    def WFD_queryMeanCount(self):

        meanCount = c.c_uint32()

        err = self._handle.LXI_PXM78xx_WFD_QueryMeanCount(self.session, self.vi, c.byref(meanCount))
        self._handleError(err)

        return meanCount.value

    def WFD_configMeanCount(self, meanCount):

        meanCount = c.c_uint32(meanCount)

        err = self._handle.LXI_PXM78xx_WFD_ConfigMeanCount(self.session, self.vi, meanCount)
        self._handleError(err)

        return

    def WFD_reset(self):

        err = self._handle.LXI_PXM78xx_WFD_Reset(self.session, self.vi)
        self._handleError(err)

        return

    def WFD_queryMaxSampleRate(self):

        fMax = c.c_uint32()

        err = self._handle.LXI_PXM78xx_WFD_QueryMaxSampleRate(self.session, self.vi, c.byref(fMax))
        self._handleError(err)

        return fMax.value

    def WFD_forceTrigger(self):

        err = self._handle.LXI_PXM78xx_WFD_ForceTrigger(self.session, self.vi)
        self._handleError(err)

        return

    def WFD_configChannel(self, channel):

        channel = c.c_int(channel)

        err = self._handle.LXI_PXM78xx_WFD_ConfigChannel(self.session, self.vi, channel)
        self._handleError(err)

        return

    def WFD_queryChannel(self):

        channel = c.c_int()

        err = self._handle.LXI_PXM78xx_WFD_QueryChannel(self.session, self.vi, c.byref(channel))
        self._handleError(err)

        return channel.value

    def WFD_configFilter(self, bwFilter):

        bwFilter = c.c_int(bwFilter)

        err = self._handle.LXI_PXM78xx_WFD_ConfigFilter(self.session, self.vi, bwFilter)
        self._handleError(err)

        return

    def WFD_queryFilter(self):

        bwFilter = c.c_int()

        err = self._handle.LXI_PXM78xx_WFD_QueryFilter(self.session, self.vi, c.byref(bwFilter))
        self._handleError(err)

        return bwFilter.value

    def WFD_configInputCoupling(self, coupling):

        coupling = c.c_int(coupling)

        err = self._handle.LXI_PXM78xx_WFD_ConfigInputCoupling(self.session, self.vi, coupling)
        self._handleError(err)

        return

    def WFD_queryInputCoupling(self):

        coupling = c.c_int()

        err = self._handle.LXI_PXM78xx_WFD_QueryInputCoupling(self.session, self.vi, c.byref(coupling))
        self._handleError(err)

        return coupling.value

    def WFD_queryFilterList(self):

        filterValuesLen = c.c_uint32(5)
        filterValues = (c.c_uint32 * filterValuesLen.value)()
        filterLabelsLen = c.c_uint32(40)
        filterLabels = c.create_string_buffer(filterLabelsLen.value)

        err = self._handle.LXI_PXM78xx_WFD_QueryFilterList(self.session,
                                                           self.vi,
                                                           c.byref(filterValues),
                                                           c.byref(filterValuesLen),
                                                           c.byref(filterLabels),
                                                           filterLabelsLen)
        self._handleError(err)

        filterValuesList = []
        for i in range(filterValuesLen.value):
            filterValuesList.append(filterValues[i])

        filterLabelsList = self._pythonString(filterLabels.value).split(";")

        filtersDict = {filterLabelsList[i]: filterValuesList[i] for i in range(len(filterValuesList))}

        return filtersDict

    def WFD_configTrigger(self, triggerSource, triggerDelay, holdOff, edge):

        triggerSource = c.c_uint64(triggerSource)
        triggerDelay = c.c_double(triggerDelay)
        holdOff = c.c_double(holdOff)
        edge = c.c_int(edge)

        err = self._handle.LXI_PXM78xx_WFD_ConfigTrigger(self.session,
                                                         self.vi,
                                                         triggerSource,
                                                         triggerDelay,
                                                         holdOff,
                                                         edge)
        self._handleError(err)

        return

    def WFD_queryTrigger(self):

        triggerSource = c.c_uint64()
        triggerDelay = c.c_double()
        holdOff = c.c_double()
        edge = c.c_int()

        err = self._handle.LXI_PXM78xx_WFD_QueryTrigger(self.session,
                                                        self.vi,
                                                        c.byref(triggerSource),
                                                        c.byref(triggerDelay),
                                                        c.byref(holdOff),
                                                        c.byref(edge))
        self._handleError(err)

        return triggerSource.value, triggerDelay.value, holdOff.value, edge.value

    def WFD_waitUntilReadyForTrigger(self, segment):

        segment = c.c_uint32(segment)

        err = self._handle.LXI_PXM78xx_WFD_WaitUntilReadyForTrigger(self.session, self.vi, segment)
        self._handleError(err)

        return

    def WFD_waitUntilFinished(self, timeout):

        timeout = c.c_double(timeout)

        err = self._handle.LXI_PXM78xx_WFD_WaitUntilFinished(self.session, self.vi, timeout)
        self._handleError(err)

        return

    def WFD_ConfigFetchTimeout(self, timeout):

        timeout = c.c_double(timeout)

        err = self._handle.LXI_PXM78xx_WFD_ConfigFetchTimeout(self.session, self.vi, timeout)
        self._handleError(err)

        return

    #endregion
    
    #region Trigger Matrix

    def TM_ConfigInputMask(self, channel, inputMask):

        channel = c.c_int(channel)
        inputMask = c.c_uint64(inputMask)

        err = self._handle.LXI_PXM78xx_TM_ConfigInputMask(self.session, self.vi, channel, inputMask)
        self._handleError(err)

        return

    def TM_QueryInputMask(self, channel):

        channel = c.c_int(channel)
        inputMask = c.c_uint32()

        err = self._handle.LXI_PXM78xx_TM_QueryInputMask(self.session, self.vi, channel, c.byref(inputMask))
        self._handleError(err)

        return inputMask.value

    def TM_ConfigInputFunction(self, channel, inputFunction):

        channel = c.c_int(channel)
        inputFunction = c.c_int(inputFunction)

        err = self._handle.LXI_PXM78xx_TM_ConfigInputFunction(self.session, self.vi, channel, inputFunction)
        self._handleError(err)

        return
        
        
    def TM_QueryInputFunction(self, channel):

        channel = c.c_int(channel)
        inputFunction = c.c_int()

        err = self._handle.LXI_PXM78xx_TM_QueryInputFunction(self.session, self.vi, channel, c.byref(inputFunction))
        self._handleError(err)

        return inputFunction.value

    def TM_ConfigInversion(self, channel, invert):

        channel = c.c_int(channel)
        invert = c.c_bool(invert)

        err = self._handle.LXI_PXM78xx_TM_ConfigInversion(self.session, self.vi, channel, invert)
        self._handleError(err)

        return

    def TM_QueryInversion(self, channel):

        channel = c.c_int(channel)
        invert = c.c_bool()

        err = self._handle.LXI_PXM78xx_TM_QueryInversion(self.session, self.vi, channel, c.byref(invert))
        self._handleError(err)

        return invert.value

    def TM_ConfigDelay(self, channel, trigDelay):

        channel = c.c_int(channel)
        trigDelay = c.c_double(trigDelay)

        err = self._handle.LXI_PXM78xx_TM_ConfigDelay(self.session, self.vi, channel, trigDelay)
        self._handleError(err)

        return
    
    def TM_QueryDelay(self, channel):

        channel = c.c_int(channel)
        trigDelay = c.c_double()

        err = self._handle.LXI_PXM78xx_TM_QueryDelay(self.session, self.vi, channel, c.byref(trigDelay))
        self._handleError(err)

        return trigDelay.value


    def TM_ConfigPulseWidth(self, channel, pulseWidth):

        channel = c.c_int(channel)
        pulseWidth = c.c_double(pulseWidth)

        err = self._handle.LXI_PXM78xx_TM_ConfigPulseWidth(self.session, self.vi, channel, pulseWidth)
        self._handleError(err)

        return

    def TM_QueryPulseWidth(self, channel):

        channel = c.c_int(channel)
        pulseWidth = c.c_double()

        err = self._handle.LXI_PXM78xx_TM_QueryPulseWidth(self.session, self.vi, channel, c.byref(pulseWidth))
        self._handleError(err)

        return pulseWidth.value


    def TM_ForceTrigger(self, affectedChannels):

        affectedChannels = c.c_uint32(affectedChannels)

        err = self._handle.LXI_PXM78xx_TM_ForceTrigger(self.session, self.vi, affectedChannels)
        self._handleError(err)

        return
    
    def TM_QueryEvent(self):

        affectedChannels = c.c_uint32()

        err = self._handle.LXI_PXM78xx_TM_QueryEvent(self.session, self.vi, c.byref(affectedChannels))
        self._handleError(err)

        return affectedChannels.value
    

    def TM_ConfigArming(self, channel, arm):

        channel = c.c_int(channel)
        arm = c.c_bool(arm)

        err = self._handle.LXI_PXM78xx_TM_ConfigArming(self.session, self.vi, channel, arm)
        self._handleError(err)

        return

    
    def TM_QueryArming(self, channel):

        channel = c.c_int(channel)
        arm = c.c_bool()

        err = self._handle.LXI_PXM78xx_TM_QueryArming(self.session, self.vi, channel, c.byref(arm))
        self._handleError(err)

        return arm.value


    def TM_ConfigSingleShot (self, channel, singleShot):
        channel = c.c_int(channel)
        singleShot = c.c_bool(singleShot)

        err = self._handle.LXI_PXM78xx_TM_ConfigSingleShot(self.session, self.vi, channel, singleShot)
        self._handleError(err)

        return

    def TM_QuerySingleShot(self, channel):

        channel = c.c_int(channel)
        singleShot = c.c_bool()

        err = self._handle.LXI_PXM78xx_TM_QuerySingleShot(self.session, self.vi, channel, c.byref(singleShot))
        self._handleError(err)

        return singleShot.value

    def TM_ConfigEdge(self, channel, edge):

        channel = c.c_int(channel)
        edge = c.c_int(edge)

        err = self._handle.LXI_PXM78xx_TM_ConfigEdge(self.session, self.vi, channel, edge)
        self._handleError(err)

        return

    def TM_QueryEdge(self, channel):

        channel = c.c_int(channel)
        edge = c.c_int()

        err = self._handle.LXI_PXM78xx_TM_QueryEdge(self.session, self.vi, channel, c.byref(edge))
        self._handleError(err)

        return edge.value
    
    def TM_ConfigMode(self, channel, mode):

        channel = c.c_int(channel)
        mode = c.c_int(mode)

        err = self._handle.LXI_PXM78xx_TM_ConfigMode(self.session, self.vi, channel, mode)
        self._handleError(err)

        return

    def TM_QueryMode(self, channel):

        channel = c.c_int(channel)
        mode = c.c_int()

        err = self._handle.LXI_PXM78xx_TM_QueryMode(self.session, self.vi, channel, c.byref(mode))
        self._handleError(err)

        return mode.value
    

    def TM_ConfigOutputEnable(self, channel, enable):

        channel = c.c_int(channel)
        enable = c.c_bool(enable)

        err = self._handle.LXI_PXM78xx_TM_ConfigOutputEnable(self.session, self.vi, channel, enable)
        self._handleError(err)

        return

    def TM_QueryOutputEnable(self, channel):

        channel = c.c_int(channel)
        enable = c.c_bool()

        err = self._handle.LXI_PXM78xx_TM_QueryOutputEnable(self.session, self.vi, channel, c.byref(enable))
        self._handleError(err)

        return enable.value

    def TM_QueryInputsAndChannels(self, inCnt, outCnt):

        inCnt = c.c_uint32(inCnt)
        outCnt = c.c_uint32(outCnt)
        inTab = (TM_INPUT_TAB * inCnt.value)()
        outTab = (TM_CHANNEL_TAB * outCnt.value)()
        
        err = self._handle.LXI_PXM78xx_TM_QueryInputsAndChannels(self.session, self.vi, c.byref(inTab), c.byref(inCnt), c.byref(outTab), c.byref(outCnt))

        self._handleError(err)

        return inTab, inCnt.value, outTab, outCnt.value
    
    def TM_Reset(self, ch):

        ch = c.c_int(ch)

        err = self._handle.LXI_PXM78xx_TM_Reset(self.session, self.vi, ch)

        self._handleError(err)

        return

    def TM_ConfigInputFilter(self, inputMask, minPulseWidth):

        inputMask = c.c_uint64(inputMask)
        minPulseWidth = c.c_double(minPulseWidth)

        err = self._handle.LXI_PXM78xx_TM_ConfigInputFilter(self.session, self.vi, inputMask, minPulseWidth)
        self._handleError(err)

        return
    
    def TM_QueryInputFilter(self, inputMask):

        inputMask = c.c_uint64(inputMask)
        minPulseWidth = c.c_double()

        err = self._handle.LXI_PXM78xx_TM_QueryInputFilter(self.session, self.vi, inputMask, c.byref(minPulseWidth))
        self._handleError(err)

        return minPulseWidth.value

    def TM_ConfigInputInversion(self, inputMask, invert):

        inputMask = c.c_uint64(inputMask)
        invert = c.c_bool(invert)

        err = self._handle.LXI_PXM78xx_TM_ConfigInputInversion(self.session, self.vi, inputMask, invert)
        self._handleError(err)

        return

    def TM_QueryInputInversion(self, inputMask):

        inputMask = c.c_uint64(inputMask)
        invert = c.c_bool()

        err = self._handle.LXI_PXM78xx_TM_QueryInputInversion(self.session, self.vi, inputMask, c.byref(invert))
        self._handleError(err)

        return invert.value

    def TM_ResetInput(self, inputMask):

        inputMask = c.c_uint64(inputMask)

        err = self._handle.LXI_PXM78xx_TM_ResetInput(self.session, self.vi, inputMask)
        self._handleError(err)

        return

    #endregion
    
    #region Timer/Counter

    
    def TC_ConfigEnable(self, ch, enable):

        ch = c.c_int(ch)
        enable = c.c_bool(enable)

        err = self._handle.LXI_PXM78xx_TC_ConfigEnable(self.session, self.vi, ch, enable)
        self._handleError(err)

        return

    def TC_QueryEnable(self, ch):

        ch = c.c_int(ch)
        enable = c.c_bool()

        err = self._handle.LXI_PXM78xx_TC_QueryEnable(self.session, self.vi, ch, c.byref(enable))
        self._handleError(err)

        return enable.value


    def TC_QueryStatus(self, ch):

        ch = c.c_int(ch)
        status = c.c_int()

        err = self._handle.LXI_PXM78xx_TC_QueryStatus(self.session, self.vi, ch, c.byref(status))
        self._handleError(err)

        return status.value

    def TC_QueryStatusMessage(self, ch):

        ch = c.c_int(ch)
        status = c.create_string_buffer(64)
        statusLen = c.c_uint32(64)

        err = self._handle.LXI_PXM78xx_TC_QueryStatusMessage(self.session, self.vi, ch, status, statusLen)
        self._handleError(err)

        return self._pythonString(status)
        

    def TC_ConfigMode(self, ch, mode):

        ch = c.c_int(ch)
        mode = c.c_int(mode)

        err = self._handle.LXI_PXM78xx_TC_ConfigMode(self.session, self.vi, ch, mode)
        self._handleError(err)

        return

    def TC_QueryMode(self, ch):

        ch = c.c_int(ch)
        mode = c.c_int()

        err = self._handle.LXI_PXM78xx_TC_QueryMode(self.session, self.vi, ch, c.byref(mode))
        self._handleError(err)

        return mode.value


    def TC_ConfigTriggerInput(self, ch, trgA, trgB):

        ch = c.c_int(ch)
        trgA = c.c_int(trgA)
        trgB = c.c_int(trgB)

        err = self._handle.LXI_PXM78xx_TC_ConfigTriggerInput(self.session, self.vi, ch, trgA, trgB)
        self._handleError(err)

        return
    
    def TC_QueryTriggerInput(self, ch):

        ch = c.c_int(ch)
        trgA = c.c_int()
        trgB = c.c_int()

        err = self._handle.LXI_PXM78xx_TC_QueryTriggerInput(self.session, self.vi, ch, c.byref(trgA), c.byref(trgB))
        self._handleError(err)

        return trgA.value, trgB.value

    def TC_ConfigGateTime(self, ch, time):

        ch = c.c_int(ch)
        time = c.c_double(time)

        err = self._handle.LXI_PXM78xx_TC_ConfigGateTime(self.session, self.vi, ch, time)
        self._handleError(err)

        return
    
    def TC_QueryGateTime(self, ch):

        ch = c.c_int(ch)
        time = c.c_double()

        err = self._handle.LXI_PXM78xx_TC_QueryGateTime(self.session, self.vi, ch, c.byref(time))
        self._handleError(err)

        return time.value


    def TC_FetchResult(self, ch, maxWaitTime):

        ch = c.c_int(ch)
        maxWaitTime = c.c_double(maxWaitTime)
        result1 = c.c_double()
        result2 = c.c_double()

        err = self._handle.LXI_PXM78xx_TC_FetchResult(self.session, self.vi, ch, c.byref(result1), c.byref(result2))
        self._handleError(err)

        return result1.value, result2.value

    def TC_Reset(self, ch):

        ch = c.c_int(ch)

        err = self._handle.LXI_PXM78xx_TC_Reset(self.session, self.vi, ch)
        self._handleError(err)

        return

    #endregion












from sys import version_info
import ctypes
import platform

__version__ = "1.1.0"

class Error(Exception):
    def __init__(self, message, errorCode=None):
        self.message = message
        self.errorCode = errorCode

    def __str__(self):
        return self.message


class Base:
    def __init__(self, session):
        self.session = session

        if platform.system() == "Windows":
            arch = platform.architecture()
            if "64bit" in arch:
                self.handle = ctypes.windll.LoadLibrary("Pi743lx_w64")

            else:
                self.handle = ctypes.windll.LoadLibrary("Pi743lx_w32")

        elif platform.system() == "Darwin":
            self.handle = ctypes.cdll.LoadLibrary("Bin/libpi743lx.so")

        elif platform.system() == "Linux":
            self.handle = ctypes.cdll.LoadLibrary("libpi743lx.so")

        self.pythonMajorVersion = version_info[0]

        self.ConnectionStatus = {
            "CON_DISCONNECT": 0,
            "CON_2WIRE_INTERNAL": 1,
            "CON_4WIRE_EXTERNAL": 2
        }

        self.OverCurrentStatus = {
            "OC_STAT_NOOC": 0,
            "OC_STAT_OCPENDING": 1,
            "OC_STAT_OCLATCHED": 2,
            "PI743LX_OC_STAT_OCLATCHPEND": 3
        }

        self.TriggerSource = {
            "TRIG_SRC_NOTRIG": 0,
            "TRIG_SRC_FRONT": 1,
            "TRIG_SRC_SOFTWARE": 2,
            "TRIG_SRC_PXI0": 3,
            "TRIG_SRC_PXI1": 4,
            "TRIG_SRC_PXI2": 5,
            "TRIG_SRC_PXI3": 6,
            "TRIG_SRC_PXI4": 7,
            "TRIG_SRC_PXI5": 8,
            "TRIG_SRC_PXI6": 9,
            "TRIG_SRC_PXI7": 10,
            "TRIG_SRC_PXISTAR": 11
        }

        self.TriggerAction = {
            "TRIG_ACT_NOACT"
            "TRIG_ACT_VC_UPDATE"
            "TRIG_ACT_V_CAPTURE"
            "TRIG_ACT_C_CAPTURE"
            "TRIG_ACT_V_UPDATE_CAPTURE"
            "TRIG_ACT_C_UPDATE_CAPTURE"
        }

        self.TriggerLevel = {
            "PI743LX_TRIG_LVL_LOW": 0,
            "PI743LX_TRIG_LVL_HIGH": 1
        }

        self.TriggerMode = {
            "PI743LX_TRIG_MODE_EDGE": 0,
            "PI743LX_TRIG_MODE_LEVEL": 1
        }

    def _stringToStr(self, inputString):
        """Take a string passed to a function in Python 2 or Python 3 and convert to
           a ctypes-friendly ASCII string"""

        # Check for Python 2 or 3
        if self.pythonMajorVersion < 3:
            if type(inputString) is str:
                return inputString
            if type(inputString) is unicode:
                return inputString.encode()
        else:
            if type(inputString) is bytes:
                return inputString
            elif type(inputString) is str:
                return inputString.encode()

    def _pythonString(self, inputString):
        """Ensure returned strings are native in Python 2 and Python 3"""

        # Check for Python 2 or 3
        if self.pythonMajorVersion < 3:
            return inputString
        else:
            return inputString.decode()

    def _handleError(self, error):
        """ Private method to raise exceptions based on error codes from driver. """
        if error:
            errorString = self.errorMessage(error)
            raise Error(errorString, errorCode=error)

    def errorMessage(self, code):
        """ Returns a string description from a given error code """
        errorString = ctypes.create_string_buffer(100)
        error = self.handle.PI743LX_ErrorCodeToMessage(self.session, int(code), ctypes.byref(errorString))
        return self._pythonString(errorString.value)

    def openCard(self, resource=None, idQuery=True, reset=True):

        # If no resource is provided open the first card found
        if resource is None:

            resources = self.findInstrumentsRsrc()

            try:
                card = Card(self.session, resources[0], idQuery, reset)
            except IndexError:
                raise Error("No 41-743 card found")

        else:
            card = Card(self.session, resource, idQuery, reset)

        return card

    def findInstrumentsRsrc(self):

        count = ctypes.c_uint32(50)
        buslist = (ctypes.c_uint32 * 50)()
        devicelist = (ctypes.c_uint32 * 50)()

        err = self.handle.PI743LX_FindInstruments(self.session,
                                                  ctypes.byref(buslist),
                                                  ctypes.byref(devicelist),
                                                  ctypes.byref(count))
        self._handleError(err)

        resources = []

        for i in range(0, count.value):
            resources.append("PXI{}::{}::INSTR".format(buslist[i], devicelist[i]))

        return resources

    def getVersion(self):

        version = ctypes.c_uint32()
        self.handle.PI743LX_GetVersion(ctypes.byref(version))

        return version.value

    def getVersionString(self):

        versionString = ctypes.create_string_buffer(100)
        err = self.handle.PI743LX_GetVersionString(ctypes.byref(versionString), 100)

        return self._pythonString(versionString.value)

    def getLastErrorCode(self):

        errorCode = ctypes.c_uint32()
        err = self.handle.PI743LX_GetLastErrorCode(self.session, ctypes.byref(errorCode))

        return errorCode.value

    def version(self):

        version = ctypes.c_uint32()

        err = self.handle.PI743LX_Version(self.session, ctypes.byref(version))
        self._handleError(err)

        return version.value


class Card(Base):
    def __init__(self, session, resource, idQuery, reset):
        Base.__init__(self, 0)

        self.session = session
        self.disposed = True

        # Card handle
        self.vi = ctypes.c_uint32(0)

        resource = self._stringToStr(resource)
        err = self.handle.PI743LX_init(self.session,
                                       resource,
                                       bool(idQuery),
                                       bool(reset),
                                       ctypes.byref(self.vi))

        self._handleError(err)

    def __del__(self):
        if not self.disposed:
            self.close()
        return

    def close(self):
        self.disposed = True
        err = self.handle.PI743LX_close(self.session, self.vi)
        self._handleError(err)

        return

    def clearOverCurrentStatus(self):
        err = self.handle.PI743LX_ClearOverCurrentStatus(self.session, self.vi)
        self._handleError(err)

        return

    def clearTriggerStatus(self):
        err = self.handle.PI743LX_ClearTriggerStatus(self.session, self.vi)
        self._handleError(err)

        return

    def getAddressCounter(self):
        addressCounter = ctypes.c_uint32()
        err = self.handle.PI743LX_GetAddressCounter(self.session, self.vi, ctypes.byref(addressCounter))
        self._handleError(err)

        return addressCounter.value

    def getClockDivider(self):
        divider = ctypes.c_uint32()

        err = self.handle.PI743LX_GetClockDivider(self.session, self.vi, ctypes.byref(divider))
        self._handleError(err)

        return divider.value

    def getMeasuredOutputCurrent(self):
        current = ctypes.c_double()

        err = self.handle.PI743LX_GetMeasuredOutputCurrent(self.session, self.vi, ctypes.byref(current))
        self._handleError(err)

        return current.value

    def getMeasuredOutputVoltage(self):
        voltage = ctypes.c_double()

        err = self.handle.PI743LX_GetMeasuredOutputVoltage(self.session, self.vi, ctypes.byref(voltage))
        self._handleError(err)

        return voltage.value

    def getOutputAll(self):
        """Returns voltage, current, power, connection status"""

        voltage = ctypes.c_double()
        current = ctypes.c_double()
        power = ctypes.c_double()
        connectionStatus = ctypes.c_uint32()

        err = self.handle.PI743LX_GetOutputAll(self.session,
                                               self.vi,
                                               ctypes.byref(voltage),
                                               ctypes.byref(current),
                                               ctypes.byref(power),
                                               ctypes.byref(connectionStatus))

        self._handleError(err)

        return voltage.value, current.value, power.value, connectionStatus.value

    def getOutputConnection(self):
        connectionStatus = ctypes.c_uint32()

        err = self.handle.PI743LX_GetOutputConnection(self.session, self.vi, ctypes.byref(connectionStatus))
        self._handleError(err)

        return connectionStatus.value

    def getOutputCurrentLimit(self):
        current = ctypes.c_double()

        err = self.handle.PI743LX_GetOutputCurrentLimit(self.session, self.vi, ctypes.byref(current))
        self._handleError(err)

        return current.value

    def getOutputVoltage(self):
        voltage = ctypes.c_double()

        err = self.handle.PI743LX_GetOutputVoltage(self.session, self.vi, ctypes.byref(voltage))
        self._handleError(err)

        return voltage.value

    def getOverCurrentStatus(self):
        overcurrent = ctypes.c_uint32()

        err = self.handle.PI743LX_GetOverCurrentStatus(self.session, self.vi, ctypes.byref(overcurrent))
        self._handleError(err)

        return overcurrent.value

    def getPowerSupplyStatus(self):
        PSUstatus = ctypes.c_uint32()

        err = self.handle.PI743LX_GetPowerSupplyStatus(self.session, self.vi, ctypes.byref(PSUstatus))
        self._handleError(err)

        return PSUstatus.value

    def getTrigger(self):
        source = ctypes.c_uint32()
        action = ctypes.c_uint32()
        level = ctypes.c_uint32()
        mode = ctypes.c_uint32()

        err = self.handle.PI743LX_GetTrigger(self.session,
                                             self.vi,
                                             ctypes.byref(source),
                                             ctypes.byref(action),
                                             ctypes.byref(level),
                                             ctypes.byref(mode))
        self._handleError(err)

        return source.value, action.value, level.value, mode.value

    def getTriggerCurrent(self):
        current = ctypes.c_double()

        err = self.handle.PI743LX_GetTriggerCurrent(self.session, self.vi, ctypes.byref(current))
        self._handleError(err)

        return current.value

    def getTriggerStatus(self):
        status = ctypes.c_uint32()

        err = self.handle.PI743LX_GetTriggerStatus(self.session, self.vi, ctypes.byref(status))
        self._handleError(err)

        return status.value

    def getTriggerVoltage(self):
        voltage = ctypes.c_double()

        err = self.handle.PI743LX_GetTriggerVoltage(self.session, self.vi, ctypes.byref(voltage))
        self._handleError(err)

        return voltage.value

    def memoryTest(self, logsize):
        errorAddress = (ctypes.c_uint32 * logsize)()
        errorData = (ctypes.c_uint32 * logsize)()
        expectData = (ctypes.c_uint32 * logsize)()
        foundErrors = (ctypes.c_uint32 * logsize)()
        logsize = ctypes.c_uint32(logsize)

        err = self.handle.PI743LX_MemoryTest(self.session,
                                             self.vi,
                                             logsize,
                                             ctypes.byref(errorAddress),
                                             ctypes.byref(errorData),
                                             ctypes.byref(expectData),
                                             ctypes.byref(foundErrors))
        self._handleError(err)

        return [e.value for e in errorAddress], [e.value for e in errorData], [e.value for e in expectData], [e.value for e in foundErrors]

    def readCapturedCurrent(self, count):

        current = (ctypes.c_double * count)()
        count = ctypes.c_uint32(count)

        err = self.handle.PI743LX_ReadCapturedCurrent(self.session, self.vi, ctypes.byref(current), count)
        self._handleError(err)

        return [c.value for c in current]

    def readCapturedVoltage(self, count):

        voltages = (ctypes.c_double * count)()
        count = ctypes.c_uint32(count)

        err = self.handle.PI743LX_ReadCapturedVoltage(self.session, self.vi, ctypes.byref(voltages), count)
        self._handleError(err)

        return [v.value for v in voltages]

    def readCurrentCalValues(self):

        offsetCurrentcdac = ctypes.c_double()
        gainCurrentcdac = ctypes.c_double()
        offsetCodecdac = ctypes.c_uint32()
        gainCodecdac = ctypes.c_uint32()

        err = self.handle.PI743LX_ReadCurrentCalValues(self.session,
                                                       self.vi,
                                                       ctypes.byref(offsetCurrentcdac),
                                                       ctypes.byref(gainCurrentcdac),
                                                       ctypes.byref(offsetCodecdac),
                                                       ctypes.byref(gainCodecdac))
        self._handleError(err)

        return offsetCurrentcdac.value, gainCurrentcdac.value, offsetCodecdac.value, gainCodecdac.value

    def readID(self):

        id = ctypes.c_uint32()

        err = self.handle.PI743LX_ReadID(self.session, self.vi, ctypes.byref(id))
        self._handleError(err)

        return id.value

    def readMaximumPower(self):

        power = ctypes.c_double()

        err = self.handle.PI743LX_ReadMaximumPower(self.session, self.vi, ctypes.byref(power))
        self._handleError(err)

        return power.value

    def readVoltageCalValues(self):

        offsetVoltagevdac = ctypes.c_double()
        gainVoltagevdac = ctypes.c_double()
        offsetCodevadc = ctypes.c_uint32()
        gainCodevadc = ctypes.c_uint32()

        err = self.handle.PI743LX_ReadVoltageCalValues(self.session,
                                                       self.vi,
                                                       ctypes.byref(offsetVoltagevdac),
                                                       ctypes.byref(gainVoltagevdac),
                                                       ctypes.byref(offsetCodevadc),
                                                       ctypes.byref(gainCodevadc))
        self._handleError(err)

        return offsetVoltagevdac.value, gainVoltagevdac.value, offsetCodevadc.value, gainCodevadc.value

    def reset(self):

        err = self.handle.PI743LX_reset(self.session, self.vi)
        self._handleError(err)

        return

    def selfTest(self):

        testResult = ctypes.c_bool()
        errorMessageSize = ctypes.c_uint32(1024)
        errorMessage = ctypes.create_string_buffer(errorMessageSize.value)

        err = self.handle.PI743LX_self_test(self.session,
                                            self.vi,
                                            ctypes.byref(testResult),
                                            ctypes.byref(errorMessage),
                                            errorMessageSize)
        self._handleError(err)

        return testResult.value, self._pythonString(errorMessage.value)

    def setAddressCounter(self, addressCounter):

        addressCounter = ctypes.c_uint32(addressCounter)

        err = self.handle.PI743LX_SetAddressCounter(self.session, self.vi, addressCounter)
        self._handleError(err)

        return

    def setClockDivider(self, divider):

        divider = ctypes.c_uint32(divider)

        err = self.handle.PI743LX_SetClockDivider(self.session, self.vi, divider)
        self._handleError(err)

        return

    def setOutputAll(self, voltage, current, power, connection):

        voltage = ctypes.c_double(voltage)
        current = ctypes.c_double(current)
        power = ctypes.c_double(power)
        connection = ctypes.c_uint32(connection)

        err = self.handle.PI743LX_SetOutputAll(self.session, self.vi, voltage, current, power, connection)
        self._handleError(err)

        return

    def setOutputConnection(self, connection):

        connection = ctypes.c_uint32(connection)

        err = self.handle.PI743LX_SetOutputConnection(self.session, self.vi, connection)
        self._handleError(err)

        return

    def setOutputCurrentLimit(self, currentLimit):

        currentLimit = ctypes.c_double(currentLimit)

        err = self.handle.PI743LX_SetOutputCurrentLimit(self.session, self.vi, currentLimit)
        self._handleError(err)

        return

    def setOutputVoltage(self, voltage):

        voltage = ctypes.c_double(voltage)

        err = self.handle.PI743LX_SetOutputVoltage(self.session, self.vi, voltage)
        self._handleError(err)

        return

    def setPowerSupplyStatus(self, status):

        status = ctypes.c_uint32(status)

        err = self.handle.PI743LX_SetPowerSupplyStatus(self.session, self.vi, status)
        self._handleError(err)

        return

    def setSoftwareTrigger(self):

        err = self.handle.PI743LX_SetSoftwareTrigger(self.session, self.vi)
        self._handleError(err)

        return

    def setTrigger(self, source, action, level, mode):

        source = ctypes.c_uint32(source)
        action = ctypes.c_uint32(action)
        level = ctypes.c_uint32(level)
        mode = ctypes.c_uint32(mode)

        err = self.handle.PI743LX_SetTrigger(self.session, self.vi, source, action, level, mode)
        self._handleError(err)

        return

    def setTriggerCurrent(self, current):

        current = ctypes.c_double(current)

        err = self.handle.PI743LX_SetTriggerCurrent(self.session, self.vi, current)
        self._handleError(err)

        return

    def setTriggerVoltage(self, voltage):

        voltage = ctypes.c_double(voltage)

        err = self.handle.PI743LX_SetTriggerVoltage(self.session, self.vi, voltage)
        self._handleError(err)

        return

    def storeCurrentCalValues(self, offsetCurrent, gainCurrent, offsetCode, gainCode):

        offsetCurrent = ctypes.c_double(offsetCurrent)
        gainCurrent = ctypes.c_double(gainCurrent)
        offsetCode = ctypes.c_uint32(offsetCode)
        gainCode = ctypes.c_uint32(gainCode)

        err = self.handle.PI743LX_StoreCurrentCalValues(self.session,
                                                        self.vi,
                                                        offsetCurrent,
                                                        gainCurrent,
                                                        offsetCode,
                                                        gainCode)
        self._handleError(err)

        return

    def storeVoltageCalValues(self, offsetVoltage, gainVoltage, offsetCode, gainCode):

        offsetVoltage = ctypes.c_double(offsetVoltage)
        gainVoltage = ctypes.c_double(gainVoltage)
        offsetCode = ctypes.c_uint32(offsetCode)
        gainCode = ctypes.c_uint32(gainCode)

        err = self.handle.PI743LX_StoreVoltageCalValues(self.session,
                                                        self.vi,
                                                        offsetVoltage,
                                                        gainVoltage,
                                                        offsetCode,
                                                        gainCode)
        self._handleError(err)

        return

    def writeMaximumPower(self, power):

        power = ctypes.c_double(power)

        err = self.handle.PI743LX_WriteMaximumPower(self.session, self.vi, power)
        self._handleError(err)

        return

    def isStabilized(self):

        stabilized = ctypes.c_bool()

        err = self.handle.PI743LX_IsStabilized(self.session, self.vi, ctypes.byref(stabilized))
        self._handleError(err)

        return

    def waitForStabilized(self, timeout):

        timeout = ctypes.c_uint32(timeout)

        err = self.handle.PI743LX_WaitForStabilized(self.session, self.vi, timeout)
        self._handleError(err)

        return

    def writeMemory(self, memoryData):

        dataLength = ctypes.c_uint32(len(memoryData))
        memoryData = (ctypes.c_double * len(memoryData))(*memoryData)

        err = self.handle.PI743LX_WriteMemory(self.session, self.vi, memoryData, dataLength)
        self._handleError(err)

        return













"""Example program for using Pickering LXI Remote AC Management Switch (power sequencer) with the Python Clientbridge wrapper."""

import pipslx
import pilxi

if __name__ == "__main__":
    print("pilxi wrapper version: {}".format(pilxi.__version__))
    # IP address of the LXI Power Management Switch
    IP_Address = "192.168.0.244"

    # Open a session with the LXI unit and get the session ID
    session = pilxi.Pi_Session(IP_Address)
    sessionID = session.GetSessionID()

    # Open the pipslx library using the LXI session
    powerSequencer = pipslx.Pipslx(sessionID)

    # Start sequence
    powerSequencer.sequence(pipslx.SEQUENCE_START)

    # Get the state of a specified channel
    channel = 2
    state = powerSequencer.get_chan_state(channel)

    # Set the state of a specified channel
    state = 1
    powerSequencer.set_chan_state(channel, state)

    emergency = False

    if emergency:
        # In an emergency, call shutdown method to immediately disconnect
        # all channels. No sequence times applied
        powerSequencer.shutdown()
    else:
        # Otherwise, begin the regular shutdown sequence
        powerSequencer.sequence(pipslx.SEQUENCE_STOP)

    # Close pipslx
    powerSequencer.close()

    # Close LXI session
    session.Close()
